/*---------------------------------------------------------------------------

	File:
		asteroids.cpp

	THIS DRIVER IS NOT PLAYABLE!!!
	
	Abstract:
		Implements the "Asteroids" game driver functions.

	Revisions:
		18-Feb-97 EAM Created

	Copyright (C) 1997 Edward Massey

---------------------------------------------------------------------------*/

#include "pch.h"
#include <stdio.h>
#include "misc.h"
#include "../inc/z80api.h"
#include "game.h"
#include "game1.h"

extern EXEC_Z80_STRUCT _es; // hack for now

namespace ns6502
{
#include "../m6502/old/6502.h"
}

extern "C"
{
void Reset6502(reg *Regs);
word M6502(reg Regs);
};

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

enum 
{
    black, red, purple, cyan, yellow, blue, green, white
};

static const int COLOR_CODES = 16; // number of color schemes

static const PALETTEENTRY _rgpe[] =
{
	{ 0x00, 0x00, 0x00 }, // black
	{ 0xFF, 0x00, 0x00 }, // red
	{ 0xB8, 0x02, 0xB8 }, // purple
	{ 0x00, 0xFF, 0xFF }, // cyan
	{ 0xFF, 0xFF, 0x00 }, // yellow
	{ 0x24, 0x24, 0xFF }, // blue
	{ 0x00, 0x90, 0x00 }, // green
	{ 0xFF, 0xFF, 0xFF }, // white
};

static const BYTE _rgbColorTable[4 * COLOR_CODES] =
{
    black, white, white, white, /* background, white text */
    black, red, blue, yellow,   /* flag ship */
    black, blue, red, yellow,   /* red bug */
    black, blue, purple, red,   /* purple bug */
    black, blue, green, red,    /* green bug */
    black, red, red, red,       /* red text */
    black, white, red, cyan,    /* player ship, cyan text */
    black, yellow, red, purple, /* explosion, purple text */
};


static HRESULT Init(GAME_LOAD_INFO &gli)
{
	static const TCHAR szPath[] = "roids";

	RAGE_ROMTABLE_BEGIN(ROMs)
		RAGE_ROMTABLE_ENTRY("035127.02", 0x5000, 0x0800)
		RAGE_ROMTABLE_ENTRY("035145.02", 0x6800, 0x0800)
		RAGE_ROMTABLE_ENTRY("035144.02", 0x7000, 0x0800)
		RAGE_ROMTABLE_ENTRY("035143.02", 0x7800, 0x0800)
		RAGE_ROMTABLE_ENTRY("035143.02", 0xf800, 0x0800)
	RAGE_ROMTABLE_END

	static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
	static const RECT _rcDisplay = { 0, 0, 248, 256 };

	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	gli.cpe = 256;
	
	for (int i = 0; i < gli.cpe; i++)
	{
		gli.ppe[i].peRed   = MulDiv(i, 256, gli.cpe);
		gli.ppe[i].peGreen = MulDiv(i, 256, gli.cpe);
		gli.ppe[i].peBlue  = MulDiv(i, 256, gli.cpe);
	}
	
//	BuildCompositePalette(_rgpe, DIM(_rgpe), _rgbColorTable,
//		COLOR_CODES * 4, gli.ppe, 256, gli.cpe);

	if (readroms(RAM, ROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	RAM[0x2001] = 0xff;
	RAM[0x2007] = 0x0;
	RAM[0x2002] = 0x0;
	RAM[0x2c00] = 0xff;
	RAM[0x2000] = 0xff;
	RAM[0x2800] = 0x0;
	RAM[0x2801] = 0x0;
	RAM[0x2802] = 0x0;
	RAM[0x2803] = 0x0;

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static BYTE M_RDMEM(WORD A)
{
	return RAM[A];
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static BYTE astLastswap;

static void M_WRMEM(WORD A, BYTE V)
{
	if (A == 0x3000)
	{
//		MessageBeep(0xFFFFFFFF);
	}
	else if (A == 0x3200)
	{
		char buffer[0x400];	/* Needed for Asteroids & Deluxe swapping */

		if ((V & 4) != astLastswap)
		{
			astLastswap = V & 0x4;
			memcpy(buffer, RAM + 0x200, 0x100);
			memcpy(RAM + 0x200, RAM + 0x300, 0x100);
			memcpy(RAM + 0x300, buffer, 0x100);
		}
	}

	RAM[A] = V;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static int Interrupt(void)
{
	if (!CheckMessages())
		CPURunning = 0;

	Throttle(60);

	return INT_NMI;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

// TODO:  call higher-level Exec function that handles the common parts
//        of Interrupt, pass it our CPU speed and frame rate

static HRESULT Run(LPVOID pvContext)
{
	reg Regs;
	
	_es.pfnReadMemory  = M_RDMEM;
	_es.pfnWriteMemory = M_WRMEM;
	_es.pfnInterrupt   = Interrupt;

	Reset6502(&Regs);
	M6502(Regs);

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

extern "C" void TRACE(LPCTSTR, ...);

static HRESULT RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
	LPBYTE pBase = &RAM[0x0000];

	int i, j;
	
	if (GetAsyncKeyState(VK_SHIFT) < 0)
	{
		for (i = 0x4000; i < 0x4010; i++)
			TRACE("%04X %02X\n", i, RAM[i]);
	}

	for (i = 0; i < 256; i++)
	{
		for (j = 0; j < 256; j++)
		{
			if (i == 64 && j > 128)
				rgbBuffer[j + i * lPitch] = 200;
			else
				rgbBuffer[j + i * lPitch] = *pBase++;

			pBase++;
		}
	}

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

GAME_DESCRIPTOR _gmdAsteroids = 
{
	TEXT("Asteroids"),
	Init,
	Run,
	RenderScene,
	NULL,
	NULL,
	NULL,
	NULL
};

