/*---------------------------------------------------------------------------

	File:
		frogger.cpp
	
	Abstract:
		Implements the Frogger game driver functions.

	Revisions:
		31-Jan-97 EAM Created
		09-Feb-97 EAM Optimizations... 120 fps (unthrottled, DirectDraw)

	Copyright (C) 1997 Edward Massey

	Hardware information gathered from Robert's Arcade Emulator.
	Information regarding the sound board was grabbed from MAME :-).

	BEWARE:  The X coordinate for the graphics has nibbles reversed!!!  The
	data lines to the first audio ROM D0 and D1 are switched!!!

---------------------------------------------------------------------------*/

#include "pch.h"
#include "misc.h"
#include "game.h"
#include "game1.h"
#include "../z80/execz80.h"
#include "../main/ay8910.h"
#include "memmap.h"

static AY8910 _ay8910;

static IMAGE *_pimgChars;
static IMAGE *_pimgSprites;

extern void sound_command_w(int offset,int data);
extern int pending_commands;
extern int sound_command_r();

/* default dip switches settings */
static BYTE _rgbDSW[] = { 0x00 };

struct SPRITE
{
	BYTE x, i, c, y;
};

struct VIDEO_ROW_ATTR
{
	BYTE bShift;
	BYTE bColor;
};

static BYTE *_pVideoRam = &RAM[0xA800];
static VIDEO_ROW_ATTR *_pVRA = (VIDEO_ROW_ATTR*)&RAM[0xB000];
static SPRITE *_pSprites = (SPRITE*)&RAM[0xB040];

struct GAME_TEMPLATE
{
	TCHAR szName[64];
	RomModule *prm;
	DECODE_GRAPHICS dcg;
};

GAME_TEMPLATE *pGameTemplates;

struct GAME_RESOURCES
{
	struct CPU_INFO
	{
		union
		{
			IExecZ80 *_pZ80;
		};

		LPBYTE rgbMemory;
	};

	CPU_INFO rgCpu[4];
	IMAGE rgpImages[16];
};

static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
static const RECT _rcDisplay = { 16, 0, 256 - 16, 256 };

static const TCHAR szPath[] = "frogger";

RAGE_ROMTABLE_BEGIN(GFXROMs)
	RAGE_ROMTABLE_ENTRY("frogger.606", 0x0000, 0x0800)
	RAGE_ROMTABLE_ENTRY("frogger.607", 0x0800, 0x0800)
RAGE_ROMTABLE_END

RAGE_ROMTABLE_BEGIN(ROMs)
	RAGE_ROMTABLE_ENTRY("frogger.ic5", 0x0000, 0x1000)
	RAGE_ROMTABLE_ENTRY("frogger.ic6", 0x1000, 0x1000)
	RAGE_ROMTABLE_ENTRY("frogger.ic7", 0x2000, 0x1000)
	RAGE_ROMTABLE_ENTRY("frogger.ic8", 0x3000, 0x1000)
RAGE_ROMTABLE_END

RAGE_ROMTABLE_BEGIN(AudioROMs)
	RAGE_ROMTABLE_ENTRY("frogger.608", 0x0000, 0x0800)
	RAGE_ROMTABLE_ENTRY("frogger.609", 0x0800, 0x0800)
	RAGE_ROMTABLE_ENTRY("frogger.610", 0x1000, 0x0800)
RAGE_ROMTABLE_END

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

enum 
{
    black, red, purple, cyan, yellow, blue, green, white
};

static const PALETTEENTRY _rgpeFrogger[] =
{
	{ 0x00, 0x00, 0x00 }, // BLACK
	{ 0xFF, 0x00, 0x00 }, // RED
	{ 0x90, 0x00, 0xDC }, // PURPLE
	{ 0x00, 0xDC, 0xDC }, // CYAN
	{ 0xFF, 0xFF, 0x00 }, // YELLOW
	{ 0x00, 0x00, 0xDC }, // BLUE
	{ 0x20, 0xDC, 0x00 }, // GREEN
	{ 0xFF, 0xFF, 0xFF }, // WHITE
	{ 0xFF, 0x40, 0x00 }, // ORANGE
	{ 0x00, 0x20, 0x60 }, // DARK BLUE
	{ 0xDC, 0x60, 0x40 }, // LIGHT BROWN
	{ 0x94, 0x60, 0x40 }, // DARK BROWN
	{ 0xFF, 0x50, 0xDC }, // PINK
};

const unsigned char _rgbFroggerColorTable[] =
{
	/* 1 UP, HI SCORE */
	black, blue, red, white,
	/* grass, CREDIT 00, fly, completed frog */
    black, 8/*orange*/, green, cyan,
	/* logs */
    black, 10/*ltbrown*/, white, 11/*drkbrown*/,
	/* frogger */ 
    black, purple, green, yellow,
    /* cars (3rd row) */
    black, green, cyan, 12/*pink*/,
    /* bulldozers (2nd row), racers (4th row), trucks (5th row), turtles 
*/     
    black, red, white, green, 
    /* middle and bottom grass, score, time (middle) */ 
    black, purple, blue, red,
    /* racers (1st row) */
    black, red, yellow, purple,
    black, red, blue, green, 	// not used
    black, red, green, blue, 	// not used
    black, red, blue, yellow, 	// not used
    black, blue, red, yellow, 	// not used
    black, blue, purple, red, 	// not used
    black, blue, green, red, 	// not used
    black, white, red, cyan,	// not used
    yellow, red, purple			// not used
};

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static IExecZ80 *_pZ80;
static IExecZ80 *_pZ80Audio;
static BYTE AudioRAM[0x10000];

static BYTE M_RDMEM(WORD A);
static void M_WRMEM(WORD A, BYTE);

static BYTE AudioM_RDMEM(WORD A);
static void AudioM_WRMEM(WORD A, BYTE);

//DWORD _dwSliceTime; // time this time-slice began
//LARGE_INTEGER _nPerfTime;

static int frogger_portB_r(/*int offset*/)
{
#if 0
	int clockticks,clock,i;

	static const int rates[8] = { 256, 2048, 0, 128, 1024, 0, 0, 0 };

	EXECZ80_CONTEXT ctx;

	_pZ80Audio->GetContext(ctx);

//	clockticks = (Z80_IPeriod - cpu_geticount());
	clockticks = (ctx.IPeriod - ctx.ICount);

/*
	DWORD dwDiffTime = timeGetTime() - _dwSliceTime;
	// clockticks is in t-states...
	clockticks = dwDiffTime * (1572750/1000);
*/

	LARGE_INTEGER x, y;
	
	if (QueryPerformanceCounter(&x) && QueryPerformanceFrequency(&y))
	{
//		clockticks = 1572750L * (x.QuadPart - _nPerfTime.QuadPart) / y.QuadPart;
//		clockticks = 2 * 1572750L * (x.QuadPart - _nPerfTime.QuadPart) / y.QuadPart;
		clockticks = (int)(x.QuadPart * 2L * 1572750L / y.QuadPart);
	}

	#define TIMER_RATE 170

	clock = (clockticks / TIMER_RATE);
	
	return clock;

	clock = 0;

	for (i = 0;i < 8;i++)
	{
		clock <<= 1;

		if (rates[i])
			clock |= ((clockticks / rates[i]) & 1);
	}

//	TRACE("clock:%i\n", clock);

	return clock;
#else
	static int clock;
	clock += 200;
	return clock;
#endif
}

/*
static unsigned char porthandler0(AY8910 *chip, int port, int iswrite, unsigned char val)
{
	int num = 0;

	if (iswrite)
	{
	}
	else
	{
		chip->Regs[port] = 0;

		if (port == 0x0e)
			chip->Regs[port] = sound_command_r(0);
		else
			chip->Regs[port] = frogger_portB_r(1);
	}

	return 0;
}
*/

BYTE FroggerAudioIn(BYTE A)
{
	switch (A)
	{
		case 0x40: // AY8910_read_port_0_r
		{
			return _ay8910.ReadRegLatch();

/*
			switch (_ay8910.ReadLatch())
			{
				case AY_PORTA:
					return sound_command_r(0);

				case AY_PORTB:
					return frogger_portB_r(1);
			
				default:
					return _ay8910.ReadRegLatch();
			}
*/
		}
	}

	return 0;
}

void FroggerAudioOut(BYTE A, BYTE data)
{
	switch (A)
	{
		case 0x40: // AY8910_write_port_0_w
			_ay8910.WriteRegLatch(data);
			break;

		case 0x80: // AY8910_control_port_0_w
			_ay8910.WriteLatch(data);
			break;
	}
}

static HRESULT Init(GAME_LOAD_INFO &gli)
{
	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	BuildCompositePalette(_rgpeFrogger, DIM(_rgpeFrogger), _rgbFroggerColorTable,
		DIM(_rgbFroggerColorTable), gli.ppe, 256, gli.cpe);

	if (readroms(RAM, ROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	if (readroms(AudioRAM, AudioROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	/* ROM 1 has data lines D0 and D1 swapped. Decode it. */
	for (int j = 0; j < 0x0800;j++)
		AudioRAM[j] = (AudioRAM[j] & 0xfc) | ((AudioRAM[j] & 1) << 1) | ((AudioRAM[j] & 2) >> 1);

	BYTE tmpstorage[0x4000];

	if (readroms(tmpstorage, GFXROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	//
	//  Decode characters
	//

	static const DECODE_GRAPHICS dgsChars =
	{
		0x00000000,
		{
			{ 8, 8 },
			256,
			8 * 8,
			2,
			{ 0, 8 * 8 * 256 },
			2,
			{ 7*8, 6*8, 5*8, 4*8, 3*8, 2*8, 1*8, 0*8 },
			{ 0, 1, 2, 3, 4, 5, 6, 7 },
		},

		1,
		
		1, 
		
		{
			0
		},
		0, 
		NULL
	};

	MyGfxDecode3(dgsChars, tmpstorage, _pimgChars);

	//
	//  Decode characters
	//

	static const DECODE_GRAPHICS dgsSprites =
	{
		0x00000001,
		{
			{ 16, 16 },
			64,
			32 * 8,
			2,
			{ 0, 64 * 16 * 16 },
			2,
			{ 23*8, 22*8, 21*8, 20*8, 19*8, 18*8, 17*8, 16*8,
				7*8, 6*8, 5*8, 4*8, 3*8, 2*8, 1*8, 0*8 },
			{ 0, 1, 2, 3, 4, 5, 6, 7,
				8*8+0, 8*8+1, 8*8+2, 8*8+3, 8*8+4, 8*8+5, 8*8+6, 8*8+7 },
		},

		1,
		
		4, 
		
		{
			0 * 64,
			1 * 64,
			2 * 64,
			3 * 64
		},
		0, 
		NULL
	};

	MyGfxDecode3(dgsSprites, tmpstorage, _pimgSprites);

	//
	//
	//

	static const EXECZ80_MEM_DESCRIPTOR rgmd[] = 
	{
		{ 0x0000, 0xFFFF, 0, NULL, NULL },
		{ 0xD000, 0xD000, 0, M_RDMEM, M_WRMEM },
		{ 0xE000, 0xE000, 0, M_RDMEM, M_WRMEM },
		{ 0xE002, 0xE002, 0, M_RDMEM, M_WRMEM },
		{ 0xE004, 0xE004, 0, M_RDMEM, M_WRMEM },
		{ 0, 0, 0, (EXECZ80_MEMREAD_PROC)0xFFFFFFFF, 0 },
	};

	EXECZ80_CREATE cs;

	ZERO_INIT(cs);

	cs.rgbMemory = RAM;
	cs.pfnMemRead = M_RDMEM;
	cs.pfnMemWrite = M_WRMEM;
	cs.pmd = rgmd;

	HRESULT hr = EclCreateZ80(cs, _pZ80);

	if (FAILED(hr))
		return hr;

	_pZ80->Reset();

	//
	//
	//
	
	static const EXECZ80_MEM_DESCRIPTOR rgmdAudio[] = 
	{
		{ 0x0000, 0xFFFF, 0, NULL, NULL    },
		{ 0x0000, 0x1FFF, 0, NULL, M_WRMEM },
		{ 0, 0, 0, (EXECZ80_MEMREAD_PROC)0xFFFFFFFF, 0 },
	};

	ZERO_INIT(cs);

	cs.rgbMemory = AudioRAM;
	cs.pfnMemRead = AudioM_RDMEM;
	cs.pfnMemWrite = AudioM_WRMEM;
	cs.pfnPortRead = FroggerAudioIn;
	cs.pfnPortWrite = FroggerAudioOut;
	cs.pmd = rgmdAudio;

	hr = EclCreateZ80(cs, _pZ80Audio);

	_ay8910.m_pfnReadPortA = sound_command_r;
	_ay8910.m_pfnReadPortB = frogger_portB_r;

//	if (AYInit(2, 1789750000, 11025, 512, 0) == 0)
	{
//		AYSetPortHandler(0,AY_PORTA,porthandler0);
//		AYSetPortHandler(0,AY_PORTB,porthandler0);
	}

	return S_OK;
}

static BYTE M_RDMEM(WORD A)
{
	switch (A)
	{
		case FROGGER_IN0_PORT:	// Switch 0
		{
			BYTE b = 0xFF;

			if (osd_key_pressed(OSD_KEY_LEFT))
				b ^= FROGGER_IN0_P1_LEFT;

			if (osd_key_pressed(OSD_KEY_RIGHT))
				b ^= FROGGER_IN0_P1_RIGHT;

			if (osd_key_pressed(OSD_KEY_3))
				b ^= FROGGER_IN0_COIN1;
			
			return b;
		}

		case FROGGER_IN1_PORT:	// Switch 1
		{
			BYTE b = 0xFF;

			if (osd_key_pressed(OSD_KEY_1))
				b ^= FROGGER_IN1_1P_START;

			if (osd_key_pressed(OSD_KEY_2))
				b ^= FROGGER_IN1_2P_START;

			return b;
		}

		case FROGGER_IN2_PORT:
		{
			BYTE b = 0xFF;

			if (osd_key_pressed(OSD_KEY_UP))
				b ^= FROGGER_IN1_P1_UP;

			if (osd_key_pressed(OSD_KEY_DOWN))
				b ^= FROGGER_IN1_P1_DOWN;

			return b;
		}

		default:
			return RAM[A];
	}
}

static void M_WRMEM(WORD A, BYTE V)
{
	if (A < 0x4000)
		return;

	switch (A)
	{
		case 0xd000:
			sound_command_w(A, V);
			break;
	}

	RAM[A] = V;
}

static int Exec()
{
	int cCycles;
	
	cCycles = _pZ80->Exec(3072000 / 60);

	if (RAM[0xB808])
		cCycles += _pZ80->Nmi();

	BOOL fSoundEnable = FALSE; // TRUE;

	if (fSoundEnable)
	{
//		_dwSliceTime = timeGetTime();

//		QueryPerformanceCounter(&_nPerfTime);

		_pZ80Audio->Exec(1572750 / 60);

		if (pending_commands > 0)
			_pZ80Audio->Irq(0xFF);
	}

	return cCycles;
}

static void Term()
{
	_pZ80->Release();
	_pZ80Audio->Release();

	FreeImage(_pimgChars);
	FreeImage(_pimgSprites);
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static void RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
	LPBYTE pBase = _pVideoRam;

	int X, Y, i;

	//
	//  Draw 32x32 characters
	//

	for (Y = 0; Y < 32; Y++)
	{
		//
		//  NOTE:  x origin and color of each line of text is set in hardware
		//

		int nShift = pBase[0x0800 + Y * 2];
		nShift = (nShift << 4 | nShift >> 4) & 0xFF;
		const int color = pBase[0x0801 + Y * 2] & 0x1F;
		const int sy = Y * 8;

		for (X = 0; X < 32; X++)
		{
			const int offset_value = pBase[(31 - X) * 32 + Y];
			const int sx = ((X * 8 + nShift) % 256);
		
			DrawChar(_pimgChars->rgbData, offset_value, color << 2,
				sx, sy, rgbBuffer, lPitch);

		}

/*
		ptr = pBase[31 * 32 + Y];

		for (X = nShift; X < 32 * 8 + nShift; X += 8)
		{
			const int offset_value = *pBase;
			const int sx = X % 256;
		
			DrawChar(_pimgChars->rgbData, offset_value, color << 2,
				sx, sy, rgbBuffer, lPitch);

			ptr += 32;
		}
*/
	}

	//
	//  Draw sprites
	//

	for (i = 0; i < 8; i++)
	{
		if (_pSprites[i].y == 0)
			continue;

		X = _pSprites[i].x;
		
		X = (X << 4 | X >> 4) & (0xff);

				// NOTE:  use of prerotated sprite images means we can pass pBase[1] unmodified
		DrawSprite(_pimgSprites->rgbData, _pimgSprites->rgbMask,
			_pSprites[i].i, _pSprites[i].c << 2, X, _pSprites[i].y, rgbBuffer, lPitch);
	}
}

static void UpdateSound(LPBYTE rgbBuffer, LONG cbBuffer)
{
//	AYSetBuffer(0, rgbBuffer);
//	AYUpdate();

	_ay8910.UpdateSound(rgbBuffer, cbBuffer);
}

static BYTE AudioM_RDMEM(WORD A)
{
	return AudioRAM[A];
}

static void AudioM_WRMEM(WORD A, BYTE B)
{
	if (A < 0x1800)
		return;

	AudioRAM[A] = B;
}

/*---------------------------------------------------------------------------

	Define game descriptors

---------------------------------------------------------------------------*/

GAME_DESCRIPTOR _gmdFrogger = 
{
	TEXT("Frogger"),
	NULL,
	Init,
	Exec,
	RenderScene,
	UpdateSound,
	Term
};

