/*---------------------------------------------------------------------------

	File:
		ladybug.cpp
	
	Abstract:
		Implements the "Ladybug" game driver functions.

	Revisions:
		01-Apr-97 EAM Created

	Copyright (C) 1997 Edward Massey

	Hardware information adopted from Nicola Salmoria's MAME.

---------------------------------------------------------------------------*/

#include "pch.h"
#include <stdio.h>
#include "misc.h"
#include "../inc/z80api.h"
#include "game.h"
#include "game1.h"

static BYTE _rgbChars[512][8][8];
static BYTE _rgbSprites[512][16][16];
static BYTE _rgbSpriteMask[512][16][16];

#include "blit.h"

static int vblank = 0;

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

// copied from MAME...

static const BYTE _rgbColorPROM[] =
{
	/* palette */
	0xF5,0x90,0x41,0x54,0x94,0x11,0x80,0x65,0x05,0xD4,0x01,0x00,0xB1,0xA0,0x00,0xF5,
	0x04,0xB1,0x00,0x15,0x11,0x25,0x90,0xD0,0xA0,0x90,0x15,0x84,0xB5,0x04,0x04,0x04,

	/* sprite color lookup table */
	0x00,0x59,0x33,0xB8,0x00,0xD4,0xA3,0x8D,0x00,0x2C,0x63,0xDD,0x00,0x22,0x38,0x1D,
	0x00,0x93,0x3A,0xDD,0x00,0xE2,0x38,0xDD,0x00,0x82,0x3A,0xD8,0x00,0x22,0x68,0x1D
};

static void BuildCompositePalette(PALETTEENTRY *ppeOut, UINT &cpeOut)
{
	for (cpeOut = 0; cpeOut < DIM(_rgbColorPROM); cpeOut++)
	{
		int i = cpeOut;

		int bit0,bit1,bit2;

		bit0 = (_rgbColorPROM[i] >> 0) & 0x01;
		bit1 = (_rgbColorPROM[i] >> 1) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 2) & 0x01;

		ppeOut[cpeOut].peRed = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;

		bit0 = (_rgbColorPROM[i] >> 3) & 0x01;
		bit1 = (_rgbColorPROM[i] >> 4) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 5) & 0x01;

		ppeOut[cpeOut].peGreen = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;

		bit0 = 0;
		bit1 = (_rgbColorPROM[i] >> 6) & 0x01;
		bit2 = (_rgbColorPROM[i] >> 7) & 0x01;

		ppeOut[cpeOut].peBlue  = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	}
}

static HRESULT Init(GAME_LOAD_INFO &gli)
{

	static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
//	static const RECT _rcDisplay = { 0, 0, 256 + 16, 256 + 16};
	static const RECT _rcDisplay = { 32, 16, 256 - 32, 256 + 16};

	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	BuildCompositePalette(gli.ppe, gli.cpe);

	//
	//
	//
	
	static const TCHAR szPath[] = "ladybug";

	RAGE_ROMTABLE_BEGIN(ROMs)
		RAGE_ROMTABLE_ENTRY("lb1.cpu", 0x0000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb2.cpu", 0x1000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb3.cpu", 0x2000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb4.cpu", 0x3000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb5.cpu", 0x4000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb6.cpu", 0x5000, 0x1000)
	RAGE_ROMTABLE_END

	if (readroms(RAM, ROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	//
	//
	//
	
	BYTE tmpstorage[0x4000];

	RAGE_ROMTABLE_BEGIN(GFXROMs)
		RAGE_ROMTABLE_ENTRY("lb9.vid", 0x0000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb10.vid",0x1000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb8.cpu", 0x2000, 0x1000)
		RAGE_ROMTABLE_ENTRY("lb7.cpu", 0x3000, 0x1000)
	RAGE_ROMTABLE_END

	if (readroms(tmpstorage, GFXROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	static const MY_GFX_DECODE mgdChars =
	{
		{ 8, 8 },
		512,
		8 * 8,
		2,
		8 * 8 * 512,
		2,
		{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
		{ 0, 1, 2, 3, 4, 5, 6, 7 },
	};

	MyGfxDecode(tmpstorage, mgdChars, (LPBYTE)_rgbChars);
	MyGfxDecode(&tmpstorage[0x1000], mgdChars, (LPBYTE)_rgbChars[256]);

	static const MY_GFX_DECODE mgdSprites =
	{
		{ 16, 16 },
		128,
		64 * 8,
		2,
		1,
		2,
		{ 23*16, 22*16, 21*16, 20*16, 19*16, 18*16, 17*16, 16*16,
			7*16, 6*16, 5*16, 4*16, 3*16, 2*16, 1*16, 0*16 },
		{ 8*16+14, 8*16+12, 8*16+10, 8*16+8, 8*16+6, 8*16+4, 8*16+2, 8*16+0,
			14, 12, 10, 8, 6, 4, 2, 0 },
	};

	static const int rgiMirror[] = {
		0 * mgdSprites.cImage,
		1 * mgdSprites.cImage,
		2 * mgdSprites.cImage,
		3 * mgdSprites.cImage
	};

	static const int cImageInc = 1;

	MyGfxDecode2(tmpstorage + 0x2000, mgdSprites, cImageInc, rgiMirror,
		(LPBYTE)_rgbSprites, (LPBYTE)_rgbSpriteMask);

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

#define IN1_VBLANK     0x10000000
#define IN1_NOT_VBLANK 0x01000000

static BYTE M_RDMEM(WORD A)
{
	switch (A)
	{
		case 0x9000:
		{
			BYTE b = 0xFF;

			if (osd_key_pressed(OSD_KEY_LEFT))
				b ^= 0x01;

			if (osd_key_pressed(OSD_KEY_DOWN))
				b ^= 0x02;

			if (osd_key_pressed(OSD_KEY_RIGHT))
				b ^= 0x04;

			if (osd_key_pressed(OSD_KEY_UP))
				b ^= 0x08;

			if (osd_key_pressed(OSD_KEY_CONTROL))
				b ^= 0x10;

			if (osd_key_pressed(OSD_KEY_1))
				b ^= 0x20;

			if (osd_key_pressed(OSD_KEY_2))
				b ^= 0x40;

			return b;
		}

		case 0x9001:
		{
			BYTE b = 0x3F;

			/* I'm not yet sure about how the vertical blanking should be handled. */
			/* I think that IN1_VBLANK should be 1 during the whole vblank, which */
			/* should last roughly 1/12th of the frame. */
			if (vblank && ICount > 50000 - (50000 / 12))
				b ^= IN1_VBLANK;
			else
			{
				b ^= IN1_NOT_VBLANK;
				vblank = 0;
			}

			return b;
		}

		case 0x9002:
		{
			BYTE b = 0xDF;

			if (osd_key_pressed(OSD_KEY_F1))
				b ^= 0x08;

			return b;
		}

		case 0x9003:
		{
			BYTE b = 0xFF;

			return b;
		}

		default:
			return RAM[A];
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static void M_WRMEM(WORD A, BYTE V)
{
	if (A < 0x6000)
		return;

	RAM[A] = V;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static int Interrupt(void)
{
	if (!CheckMessages())
		CPURunning = 0;

	vblank = 1;

	if (osd_key_pressed(OSD_KEY_3))
		return 0xFF;

	Throttle(60); // FRAMES_PER_SECOND);

	return IGNORE_INT;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

// TODO:  call higher-level Exec function that handles the common parts
//        of Interrupt, pass it our CPU speed and frame rate

static HRESULT Run(LPVOID pvContext)
{
	EXEC_Z80_STRUCT es;
	
	ZERO_INIT(es);
	
	es.nCallbackFreq = 66000; // CALLBACK_FREQ;
	es.pfnInterrupt = Interrupt;
	es.pfnReadMemory = M_RDMEM;
	es.pfnWriteMemory = M_WRMEM;
	es.rgbMemory = RAM;
	
	return ExecZ80(es);
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static HRESULT RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
	int i, j;

/*
	{ 0xd000, 0xd3ff, videoram_w, &videoram },
	{ 0xd400, 0xd7ff, colorram_w, &colorram },
	{ 0x7000, 0x73ff, MWA_RAM, &spriteram },
*/
	for (i = 0; i < 32 * 32; i++)
	{
		DrawChar((LPBYTE)_rgbChars, RAM[0xD000 + i] + 32 * (RAM[0xD400 + i] & 0x08),
			RAM[0xD400 + i], 8 * (i / 32), 8 * (31 - (i % 32)), rgbBuffer, lPitch);
	}

	LPBYTE spriteram = &RAM[0x7000];

#if 0
	for (offs = 14 * 0x40;offs >= 2;offs -= 0x40)
	{
		i = 0;
		while (i < 0x40 && spriteram[offs + i] != 0)
			i += 4;

		while (i > 0)
		{
/*
 aabbcccc ddddddee fffghhhh iiiiiiii

 aa unknown
 bb flip
 cccc x offset
 dddddd sprite code
 ee unknown
 fff unknown
 g sprite bank
 hhhh color
 iiiiiiii y position
*/
			i -= 4;

			drawgfx(bitmap,Machine->gfx[1],
					(spriteram[offs + i + 1] >> 2) + 4 * (spriteram[offs + i + 2] & 0x10),
					spriteram[offs + i + 2] & 0x0f,
					spriteram[offs + i] & 0x10,spriteram[offs + i] & 0x20,
					offs / 4 - 8 + (spriteram[offs + i] & 0x0f),
					240 - spriteram[offs + i + 3],
					&Machine->drv->visible_area,TRANSPARENCY_PEN,0);


			DrawSprite(rgbBuffer, lPitch, 255 - rgs[i].x, 255 - rgs[i].y,
				(rgs[i].i & 0x7F) + ((rgs[i].c & 0x30) * 8), (rgs[i].c & 0x0F) << 2);
		}
	}
#endif

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

GAME_DESCRIPTOR _gmdLadybug = 
{
	TEXT("Ladybug"),
	Init,
	Run,
	RenderScene,
	NULL,
	NULL,
	NULL,
	NULL
};

