/*---------------------------------------------------------------------------

	File:
		MoonPatrol.cpp
	
	Abstract:
		Implements the "Moon Patrol" game driver functions.

	Revisions:
		24-Mar-97 EAM Created

	Copyright (C) 1997 Edward Massey

	Hardware information was adopted from MAME, by Nicola Salmoria.

	TODO: background chars

---------------------------------------------------------------------------*/

#include "pch.h"
#include "misc.h"
#include "game.h"
#include "game1.h"
#include "../z80/execz80.h"

static IMAGE *_pimgSprites;
static IMAGE *_pimgChars;
static BYTE _bScroll;
static IExecZ80 *_pZ80;

RAGE_ROMTABLE_BEGIN(ROMs)
	RAGE_ROMTABLE_ENTRY("mp-a.3m", 0x0000, 0x1000)
	RAGE_ROMTABLE_ENTRY("mp-a.3l", 0x1000, 0x1000)
	RAGE_ROMTABLE_ENTRY("mp-a.3k", 0x2000, 0x1000)
	RAGE_ROMTABLE_ENTRY("mp-a.3j", 0x3000, 0x1000)
RAGE_ROMTABLE_END

RAGE_ROMTABLE_BEGIN(CharROMs)
	RAGE_ROMTABLE_ENTRY("mp-e.3e", 0x0000, 0x1000)
	RAGE_ROMTABLE_ENTRY("mp-e.3f", 0x1000, 0x1000)
RAGE_ROMTABLE_END

RAGE_ROMTABLE_BEGIN(SpriteROMs)
	RAGE_ROMTABLE_ENTRY("mp-b.3m", 0x2000, 0x1000)
	RAGE_ROMTABLE_ENTRY("mp-b.3n", 0x3000, 0x1000)
RAGE_ROMTABLE_END

RAGE_ROMTABLE_BEGIN(BkgndROMs)
	RAGE_ROMTABLE_ENTRY("mp-e.3h", 0x4000, 0x1000)
	RAGE_ROMTABLE_ENTRY("mp-e.3k", 0x5000, 0x1000)
	RAGE_ROMTABLE_ENTRY("mp-e.3l", 0x6000, 0x1000)
RAGE_ROMTABLE_END

static const TCHAR szPath[] = "mpatrol";
static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
//static const RECT _rcDisplay = { 0, 0, 256 + 16, 256 + 16};
static const RECT _rcDisplay = { 8, 8, 256 -8, 256 };

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static const PALETTEENTRY _rgpe[] =
{
	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0xd8, 0x00, 0x00 },   /* darkred    */
	{ 0x00, 0x00, 0xd8 },   /* blue   */
	{ 0xd8, 0xd8, 0x94 },   /* darkyellow */

	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0x00, 0xf8, 0x00 },   /* green  */
	{ 0x94, 0x00, 0xd8 },   /* darkpurple */
	{ 0xf8, 0x94, 0x44 },   /* orange     */

	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0x00, 0xd8, 0x00 },   /* darkgreen  */
	{ 0xd8, 0x00, 0x00 },   /* darkred    */
	{ 0xf8, 0xf8, 0x00 },   /* yellow */

	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0xd8, 0x00, 0x00 },   /* darkred    */
	{ 0x00, 0xd8, 0x00 },   /* darkgreen  */
	{ 0xf8, 0xf8, 0x00 },   /* yellow */

	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0xf8, 0xf8, 0x00 },   /* yellow */
	{ 0x00, 0xd8, 0x00 },   /* darkgreen  */
	{ 0xf8, 0x00, 0x00 },   /* red    */

	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0x00, 0xf8, 0x00 },   /* green  */
	{ 0xf8, 0x94, 0x44 },   /* orange     */
	{ 0xf8, 0xf8, 0x00 },   /* yellow */

	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0xd8, 0xf8, 0xd8 },   /* darkwhite  */
	{ 0xf8, 0x00, 0x00 },   /* red    */
	{ 0xf8, 0x64, 0xd8 },   /* pink       */

	{ 0x00, 0x00, 0x00 },   /* black      */
	{ 0x00, 0xf8, 0xd8 },   /* darkcyan   */
	{ 0xf8, 0x00, 0x00 },   /* red    */
	{ 0xd8, 0xf8, 0xd8 },   /* darkwhite  */
};

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static BYTE M_RDMEM(WORD A)
{
	switch (A)
	{
		case 0xD000:
		{
			BYTE b = 0xFF;

			if (osd_key_pressed(OSD_KEY_1))
				b ^= 0x01;

			if (osd_key_pressed(OSD_KEY_2))
				b ^= 0x02;

			if (osd_key_pressed(OSD_KEY_3))
				b ^= 0x08;

			return b;
		}
			
		case 0xD001:
		{
			BYTE b = 0xFF;

			if (osd_key_pressed(OSD_KEY_RIGHT))
				b ^= 0x01;

			if (osd_key_pressed(OSD_KEY_LEFT))
				b ^= 0x02;

			if (osd_key_pressed(OSD_KEY_ALT))
				b ^= 0x20;

			if (osd_key_pressed(OSD_KEY_CONTROL))
				b ^= 0x80;

			return b;
		}
			
		case 0xD002:
		{
			BYTE b = 0xFF;

			return b;
		}
			
		case 0xD003:
		{
			BYTE b = 0xFD;

			return b;
		}
			
		case 0xD004:
		{
			BYTE b = 0xFD;

			if (osd_key_pressed(OSD_KEY_F2))
				b ^= 0x80;

			return b;
		}
			
		case 0x8800:
		{
			EXECZ80_CONTEXT ctx;
			
			_pZ80->GetContext(ctx);

//			return R.DE.B.l;
			return LOBYTE(ctx.DE);
		}
			
		default:
			return RAM[A];
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static void M_WRMEM(WORD A, BYTE V)
{
	if (A < 0x4000)
		return;

	RAM[A] = V;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static void Out(BYTE bAddr, BYTE bVal)
{
	switch (bAddr)
	{
		case 0x1C:
		{
			_bScroll = bVal;

			break;
		}

		case 0x1D:
		case 0x1E:
		case 0x1F:
		{
			break;
		}

		case 0x40:
		{
			break;
		}

		case 0x80:
		{
			break;
		}
	}
}

static HRESULT Init(GAME_LOAD_INFO &gli)
{
	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	for (int i = 0; i < DIM(_rgpe); i++)
		gli.ppe[i] = _rgpe[i];

	gli.cpe = i;

	//
	//
	//

	if (readroms(RAM, ROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	//
	//
	//
	
	BYTE tmpstorage[0x7000];

	//
	//
	//

	DECODE_GRAPHICS dgsChars =
	{
		0x00000000,
		{
			{ 8, 8 },
			512,
			8 * 8,
			2,
			{ 512 * 8 * 8 },
			2,
			{ 0, 1, 2, 3, 4, 5, 6, 7 },
			{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
		},

		1,
		
		1, 
		
		{
			0
		},
		0, 
		NULL
	};

	if (readroms(tmpstorage, CharROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	MyGfxDecode3(dgsChars, tmpstorage, _pimgChars);

	//
	//  Decode sprites
	//
	
	DECODE_GRAPHICS dgs =
	{
		0x00000001,
		{
			{ 16, 16 },
			128,
			32 * 8,
			2,
			{ 0, 128 * 16 * 16 },
			2,
			{ 0, 1, 2, 3, 4, 5, 6, 7,
					16*8+0, 16*8+1, 16*8+2, 16*8+3, 16*8+4, 16*8+5, 16*8+6, 16*8+7 },
			{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8,
					8*8, 9*8, 10*8, 11*8, 12*8, 13*8, 14*8, 15*8 },
		},

		1,
		
		4, 
		
		{
			0 * 128,
			1 * 128,
			2 * 128,
			3 * 128
		},
		0, 
		NULL
	};

	if (readroms(tmpstorage, SpriteROMs, szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	MyGfxDecode3(dgs, tmpstorage + 0x2000, _pimgSprites);

	//
	//
	//
	
	static const EXECZ80_MEM_DESCRIPTOR rgmd[] = 
	{
//		{ 0x0000, 0xFFFF, 0, NULL, NULL },
		{ 0x0000, 0xFFFF, 0, M_RDMEM, M_WRMEM },
/*
		{ 0x7C00, 0x7C00, 0, 
		{ 0x7C80, 0x7C80, 0, M_RDMEM, M_WRMEM },
		{ 0x7D00, 0x7D07, 0, M_RDMEM, M_WRMEM },
		{ 0x7D80, 0x7D80, 0, M_RDMEM, M_WRMEM },
*/
		{ 0, 0, 0, (EXECZ80_MEMREAD_PROC)0xFFFFFFFF, 0 },
	};

	EXECZ80_CREATE cs;

	ZERO_INIT(cs);

	cs.rgbMemory = RAM;
	cs.pfnMemRead = M_RDMEM;
	cs.pfnMemWrite = M_WRMEM;
	cs.pfnPortWrite = Out;
	cs.pmd = rgmd;
	
	HRESULT hr = EclCreateZ80(cs, _pZ80);

	if (FAILED(hr))
		return hr;

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

extern void SoundUpdateMixer(LPBYTE rgbBuffer, LONG cbBuffer);

static void UpdateSound(LPBYTE rgbBuffer, LONG cbBuffer)
{
	SoundUpdateMixer(rgbBuffer, cbBuffer);
}

static int Exec()
{
	int cCycles;
	
	cCycles = _pZ80->Exec(50000);
	
	cCycles += _pZ80->Irq(0xFF);

	return cCycles;
}

static void Term()
{
	_pZ80->Release();

	FreeImage(_pimgSprites);
	FreeImage(_pimgChars);
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static void RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
	int i, j;

#if 0
	if (GetAsyncKeyState('C') < 0)
	{
		for (i = 0; i < 512; i++)
			DrawChar(rgbBuffer, lPitch, 8 * (i % 32), 8 * (i / 32), i, 0);

		return S_OK;
	}

	if (GetAsyncKeyState('S') < 0)
	{
		for (i = 0; i < 128; i++)
			DrawSprite(rgbBuffer, lPitch, 16 * (i % 16), 16 * (i / 16), i, 0);

		return S_OK;
	}
#endif

	//
	//
	//
	
	for (i = 0; i < 24; i++)
	{
		for (j = 0; j < 32; j++)
		{
			int cb = i * 32 + j;

//			DrawChar((LPBYTE)_rgbChars, RAM[0x8000 + cb] + 2 * (RAM[0x8400 + cb] & 0x80),
//				(RAM[0x8400 + cb] & 0x7F) << 2, 8 * j, 8 * i, rgbBuffer, lPitch);

//			DrawChar(_pimgChars->rgbData, RAM[0x8000 + cb] + 2 * (RAM[0x8400 + cb] & 0x80),
//				(RAM[0x8400 + cb] & 0x7F) << 2, 8 * j, 8 * i, rgbBuffer, lPitch);

			_pimgChars->DrawChar(rgbBuffer, lPitch, 8 * j, 8 * i, RAM[0x8000 + cb] + 2 * (RAM[0x8400 + cb] & 0x80), (RAM[0x8400 + cb] & 0x7F) << 2);
		}
	}

	for (i = 24; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			int cb = i * 32 + j;

//			DrawChar(_pimgChars->rgbData, RAM[0x8000 + cb] + 2 * (RAM[0x8400 + cb] & 0x80),
//				(RAM[0x8400 + cb] & 0x7F) << 2, (_bScroll + (8 * j)) % 256, 8 * i,
//				rgbBuffer, lPitch);

			_pimgChars->DrawChar(rgbBuffer, lPitch,
				(_bScroll + (8 * j)) % 256, 8 * i,
				RAM[0x8000 + cb] + 2 * (RAM[0x8400 + cb] & 0x80),
				(RAM[0x8400 + cb] & 0x7F) << 2);
		}
	}

	//
	//
	//
	
	struct SPRITE
	{
		BYTE y, c, i, x;
	};

	const SPRITE *ps = (SPRITE*)&RAM[0xC8A0];
	
	for (i = 23; i >= 0; i--)
	{
		DrawSprite(_pimgSprites->rgbData, _pimgSprites->rgbMask,
			ps[i].i + ((ps[i].c & 0xC0) * 2), (ps[i].c & 0x3F) << 2,
			ps[i].x, 241 - ps[i].y,
			rgbBuffer, lPitch);
	}

	ps = (SPRITE*)&RAM[0xC820];

	for (i = 23; i >= 0; i--)
	{
		DrawSprite(_pimgSprites->rgbData, _pimgSprites->rgbMask,
			ps[i].i + ((ps[i].c & 0xC0) * 2), (ps[i].c & 0x3F) << 2,
			ps[i].x, 241 - ps[i].y,
			rgbBuffer, lPitch);
	}
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

GAME_DESCRIPTOR _gmdMoonPatrol = 
{
	TEXT("Moon Patrol"),
	NULL,
	Init,
	Exec,
	RenderScene,
	UpdateSound,
	Term
};

