/*---------------------------------------------------------------------------

	File:
		sound.cpp

	THIS MODULE IS CURRENTLY OF VERY LITTLE USE TO MAGEX.  IT IS PART OF THE
	SOUND GENERATOR LIBRARY.  CURRENTLY UNDER DEVELOPMENT.

	Abstract:
		Sound generator for MageX.
		
	Revisions:
		14-Feb-97 EAM Created

---------------------------------------------------------------------------*/

#include "pch.h"

struct CHANNEL
{
	int iEnv;
	int vol;
	int freq;
};

#define ENV_COUNT 32
#define BUFFER_SIZE 1024
#define CHANNEL_COUNT 32

static BYTE _rgbEnvelopes[ENV_COUNT][BUFFER_SIZE];
static BYTE _rgbBuffer[BUFFER_SIZE];
static CHANNEL _rgChannel[CHANNEL_COUNT];
static int _cChannel;
static const clip = (2 << 8) - 1;

static void UpdateBuffer()
{
	for (int i = 0; i < BUFFER_SIZE; i++)
	{
		int acc = 0;

		for (int j = 0; j < _cChannel; j++)
		{
			acc += _rgbEnvelopes[_rgChannel[j].iEnv][(i * _rgChannel[j].freq / 256) % 32] *
				_rgChannel[j].vol;
		}
			
		_rgbBuffer[i] = (acc > clip) ? clip : acc;
	}
}

HRESULT SoundInit()
{
	return E_NOTIMPL;
}

HRESULT SoundTerm()
{
	return E_NOTIMPL;
}

HRESULT SoundPause()
{
	return E_NOTIMPL;
}

HRESULT SoundResume()
{
	return E_NOTIMPL;
}

/*---------------------------------------------------------------------------

Little game driver to test the sound generator

---------------------------------------------------------------------------*/

#include "../inc/z80api.h"
#include "game.h"
#include "game1.h"
#include "misc.h"
#include "stdarg.h"

static const TCHAR _szPath[] = "galaxian";

RAGE_ROMTABLE_BEGIN(GFXROMs)
	RAGE_ROMTABLE_ENTRY("galaxian.1j", 0x0000, 0x0800)
	RAGE_ROMTABLE_ENTRY("galaxian.1k", 0x0800, 0x0800)
RAGE_ROMTABLE_END

static BYTE _rgbChars[256][8][8];
static BYTE _rgbSprites[256][8][8];
static BYTE _rgbSpriteMask[256][8][8];

#include "blit.h"

enum 
{
    black, red, purple, cyan, yellow, blue, green, white
};

static const int COLOR_CODES = 16; // number of color schemes

static const PALETTEENTRY _rgpe[] =
{
	{ 0x00, 0x00, 0x00 }, // black
	{ 0xFF, 0x00, 0x00 }, // red
	{ 0xB8, 0x02, 0xB8 }, // purple
	{ 0x00, 0xFF, 0xFF }, // cyan
	{ 0xFF, 0xFF, 0x00 }, // yellow
	{ 0x24, 0x24, 0xFF }, // blue
	{ 0x00, 0x90, 0x00 }, // green
	{ 0xFF, 0xFF, 0xFF }, // white
};

static const BYTE _rgiColorTable[4 * COLOR_CODES] =
{
    black, white, white, white, /* background, white text */
    black, red, blue, yellow,   /* flag ship */
    black, blue, red, yellow,   /* red bug */
    black, blue, purple, red,   /* purple bug */
    black, blue, green, red,    /* green bug */
    black, red, red, red,       /* red text */
    black, white, red, cyan,    /* player ship, cyan text */
    black, yellow, red, purple, /* explosion, purple text */
};

static HRESULT Init(GAME_LOAD_INFO &gli)
{
	static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
	static const RECT _rcDisplay = { 0, 0, 256, 256 };

	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	BuildCompositePalette(_rgpe, DIM(_rgpe), _rgiColorTable,
		COLOR_CODES * 4, gli.ppe, 256, gli.cpe);

	unsigned char tmpstorage[0x1000];

	if (readroms(tmpstorage, GFXROMs, _szPath, gli.pszMsg, gli.cchMsgMax) != 0)
		return E_FAIL;

	MY_GFX_DECODE mgd = {
		{ 8, 8 },
		256,
		8 * 8,
		2,
		8 * 8 * 256,
		2,
		{ 7*8, 6*8, 5*8, 4*8, 3*8, 2*8, 1*8, 0*8 },
		{ 0, 1, 2, 3, 4, 5, 6, 7 },
	};

	MyGfxDecode(tmpstorage, mgd, (LPBYTE)_rgbChars);

	return S_OK;
}

static BYTE M_RDMEM(WORD A)
{
	return RAM[A];
}

static void M_WRMEM(WORD A, BYTE V)
{
	RAM[A] = V;
}

static const unsigned char samples[8][32] =
{
	{ 0xff,0x11,0x22,0x33,0x44,0x55,0x55,0x66,0x66,0x66,0x55,0x55,0x44,0x33,0x22,0x11,
	0xff,0xdd,0xcc,0xbb,0xaa,0x99,0x99,0x88,0x88,0x88,0x99,0x99,0xaa,0xbb,0xcc,0xdd },
	{ 0xff,0x44,0x66,0x66,0x55,0x33,0x11,0x22,0x33,0x33,0x22,0x11,0xee,0xcc,0xbb,0xdd,
	0xff,0x11,0x33,0x22,0x00,0xdd,0xcc,0xbb,0xbb,0xcc,0xdd,0xbb,0x99,0x88,0x88,0xaa },
	{ 0xff,0x22,0x44,0x55,0x66,0x55,0x44,0x22,0xff,0xcc,0xaa,0x99,0x88,0x99,0xaa,0xcc,
	0xff,0x33,0x55,0x66,0x55,0x33,0xff,0xbb,0x99,0x88,0x99,0xbb,0xff,0x66,0xff,0x88 },
	{ 0xff,0x55,0x33,0x00,0x33,0x55,0x11,0xee,0x33,0x66,0x44,0xff,0x11,0x22,0xee,0xaa,
	0xff,0x44,0x00,0xcc,0xdd,0xff,0xaa,0x88,0xbb,0x00,0xdd,0x99,0xbb,0xee,0xbb,0x99 },
	{ 0x88,0x00,0x77,0xff,0x99,0x00,0x66,0xff,0xaa,0x00,0x55,0xff,0xbb,0x00,0x44,0xff,
	0xcc,0x00,0x33,0xff,0xdd,0x00,0x22,0xff,0xee,0x00,0x11,0xff,0xff,0x00,0x00,0xff },
	{ 0xff,0x00,0xee,0x11,0xdd,0x22,0xcc,0x33,0xbb,0x44,0xaa,0x55,0x99,0x66,0x88,0x77,
	0x88,0x77,0x99,0x66,0xaa,0x55,0xbb,0x44,0xcc,0x33,0xdd,0x22,0xee,0x11,0xff,0x00 },
	{ 0x88,0x99,0xaa,0xbb,0xcc,0xdd,0xee,0xff,0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,
	0x77,0x66,0x55,0x44,0x33,0x22,0x11,0x00,0xff,0xee,0xdd,0xcc,0xbb,0xaa,0x99,0x88 },
	{ 0x88,0x99,0xaa,0xbb,0xcc,0xdd,0xee,0xff,0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,
	0x88,0x99,0xaa,0xbb,0xcc,0xdd,0xee,0xff,0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77 }
};

struct CHNL
{
	int iFreq;
	int iEnv;
	int nVol;
};

static CHNL _rgChnl[2] = {
	{ 0, 0, 255 },
	{ 0, 0, 128 },
};

static int _iChnl;

static BYTE _rgbScreen[32][32] = {
	{ "Sound Synthesizer Test         " },
	{ "                               " },
	{ "Chnl Freq Env  Vol             " },
	{ "1                              " },
	{ "2                              " },
	{ "                               " },
	{ "Keys:                          " },
	{ "  Q - select channel 1         " },
	{ "  W - select channel 2         " },
	{ "  A - increase freq            " },
	{ "  Z - decrease freq            " },
	{ "  0-7 - select envelope        " },
	{ "                               " },
	{ "                               " },
	{ "                               " },
	{ "                               " },
};

static BYTE _rgbColors[32][32] = {
	0
};

void text(int i, int j, LPCTSTR pszFmt, ...)
{
	TCHAR szBuf[32];
	LPCTSTR pszBuf = szBuf;
	va_list marker;
	va_start(marker, pszFmt);
	vsprintf(szBuf, pszFmt, marker);
	while (*pszBuf != 0)
	{
		_rgbScreen[i][j] = *pszBuf;
		pszBuf++;
		j++;
	}
}

static int Interrupt(void)
{
	if (!CheckMessages())
		CPURunning = 0;

	if (GetAsyncKeyState('Q') < 0)
		_iChnl = 0;
	else if (GetAsyncKeyState('W') < 0)
		_iChnl = 1;

	if (GetAsyncKeyState('A') < 0)
		_rgChnl[_iChnl].iFreq += 1;
	else if (GetAsyncKeyState('Z') < 0)
		_rgChnl[_iChnl].iFreq -= 1;

	if (GetAsyncKeyState('S') < 0)
		_rgChnl[_iChnl].nVol += 1;
	else if (GetAsyncKeyState('X') < 0)
		_rgChnl[_iChnl].nVol -= 1;

	for (int iEnv = 0; iEnv < 8; iEnv++)
	{
		if (GetAsyncKeyState('0' + iEnv) < 0)
		{
			_rgChnl[_iChnl].iEnv = iEnv;
			break;
		}
	}

	text(3, 5, "%i  ", _rgChnl[0].iFreq);
	text(3, 9, "%i  ", _rgChnl[0].iEnv);
	text(3, 13, "%i  ", _rgChnl[0].nVol);
	text(4, 5, "%i  ", _rgChnl[1].iFreq);
	text(4, 9, "%i  ", _rgChnl[1].iEnv);
	text(4, 13, "%i  ", _rgChnl[1].nVol);
	
/*
	for (int i = 0; i < DIM(_rgwPcmBuf); i++)
	{
		int v = 0;
		
//		v += (samples[_rgChnl[0].iEnv][(i * _rgChnl[0].iFreq / 256) % 32] * _rgChnl[0].nVol) / 256;
//		v += (samples[_rgChnl[1].iEnv][(i * _rgChnl[1].iFreq / 256) % 32] * _rgChnl[1].nVol) / 256;
		v += samples[_rgChnl[0].iEnv][(i * _rgChnl[0].iFreq / 256) % 32] / 4;
		v += samples[_rgChnl[1].iEnv][(i * _rgChnl[1].iFreq / 256) % 32] / 4;

		if (v > 255) v = 255;

		_rgwPcmBuf[i] = v;
	}
*/

	Throttle(60);

	return IGNORE_INT;
}

// TODO:  call higher-level Exec function that handles the common parts
//        of Interrupt, pass it our CPU speed and frame rate

static HRESULT Run(LPVOID pvContext)
{
	EXEC_Z80_STRUCT es;
	
	ZERO_INIT(es);
	
	es.nCallbackFreq = 50000;
	es.pfnInterrupt = Interrupt;
	es.pfnReadMemory = M_RDMEM;
	es.pfnWriteMemory = M_WRMEM;
	es.rgbMemory = RAM;
	
	return ExecZ80(es);
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static HRESULT UpdateSound(LPBYTE rgbBuffer, LONG cbBuffer)
{
	for (int i = 0; i < cbBuffer; i++)
	{
		int v = 0;
		
//		v += (samples[_rgChnl[0].iEnv][(i * _rgChnl[0].iFreq / 256) % 32] * _rgChnl[0].nVol) / 256;
//		v += (samples[_rgChnl[1].iEnv][(i * _rgChnl[1].iFreq / 256) % 32] * _rgChnl[1].nVol) / 256;
		v += samples[_rgChnl[0].iEnv][(i * _rgChnl[0].iFreq / 256) % 32] / 4;
		v += samples[_rgChnl[1].iEnv][(i * _rgChnl[1].iFreq / 256) % 32] / 4;

		if (v > 255) v = 255;

		rgbBuffer[i] = v;
	}

	return S_OK;
}

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*/

static HRESULT RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
	//
	//  Draw 32x32 characters
	//

	int i, j, k, l;

	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			k = _rgbScreen[i][j];
			l = _rgbColors[i][j];

			if (k >= 'a' && k <= 'z')
				k = 17 + k - 'a';
			else if (k >= 'A' && k <= 'Z')
				k = 17 + k - 'A';
			else if (k >= '0' && k <= '9')
				k = k - '0';
			else
				k = 26 + 16 + 2;

			DrawChar((LPBYTE)_rgbChars, k, l, 8 * j, 8 * i, rgbBuffer, lPitch);
		}
	}

/*
	for (i = 0; i < DIM(_rgwPcmBuf); i++)
	{
		rgbBuffer[lPitch * _rgwPcmBuf[i] + ((i * 256) / DIM(_rgwPcmBuf))] = 6;
	}
*/

	return S_OK;
}

GAME_DESCRIPTOR _gmdSoundTest = 
{
	TEXT("PSG Tester"),
	Init,
	Run,
	RenderScene,
	NULL,
	NULL,
	NULL,
	UpdateSound
};

