/*---------------------------------------------------------------------------

	File:
		testz80.cpp
	
	Abstract:

	Revisions:
		30-Apr-97 EAM Created

	Copyright (C) 1997 Edward Massey

---------------------------------------------------------------------------*/

#include "pch.h"
#include "misc.h"
#include "game.h"
#include "game1.h"
#include "../z80/execz80.h"

extern "C" __declspec(dllimport) int Z80_Dasm (unsigned char *buffer,char *dest,
	unsigned PC);

static IExecZ80 *_pZ80;

static BYTE M_RDMEM(WORD A);
static void M_WRMEM(WORD A, BYTE);

static HRESULT Init(GAME_LOAD_INFO &gli)
{
	ZERO_INIT(RAM); // fill RAM with NOPs

	static const SIZE _sizBuffer = { 256 + 16, 256 + 16};
	static const RECT _rcDisplay = { 16, 0, 256 - 16, 256 };

	gli.sizBuffer = _sizBuffer;
	gli.rcDisplay = _rcDisplay;

	//
	//
	//
	
	EXECZ80_CREATE cs;

	ZERO_INIT(cs);

	static const EXECZ80_MEM_DESCRIPTOR rgmd[] = 
	{
		{ 0x0000, 0xFFFF, 0, M_RDMEM, M_WRMEM },
		{ 0, 0, 0, (EXECZ80_MEMREAD_PROC)0xFFFFFFFF, 0 },
	};

	cs.rgbMemory = RAM;
	cs.pfnMemRead = M_RDMEM;
	cs.pfnMemWrite = M_WRMEM;
	cs.pmd = rgmd;
	
	HRESULT hr = EclCreateZ80(cs, _pZ80);

	if (FAILED(hr))
		return hr;

	_pZ80->Reset();

	return S_OK;
}

static BYTE M_RDMEM(WORD A)
{
	TRACE("RAM[%04X]\n", A);

	return RAM[A];
}

static void M_WRMEM(WORD A, BYTE V)
{
	TRACE("RAM[%04X]=%02X\n", A, V);

	RAM[A] = V;
}

static void UpdateSound(LPBYTE rgbBuffer, LONG cbBuffer)
{
}

static int Exec()
{
	static WORD wCode = 0;

	//
	//
	//
	
	EXECZ80_CONTEXT ctx;

	_pZ80->GetContext(ctx);

	if (wCode == 0)
	{
		ctx.BC = 0x1122;
		ctx.DE = 0x3344;
		ctx.HL = 0x5566;

		TRACE("PC   AF   BC   DE   HL   IX   IY   SP        Dasm\n");
		TRACE("---- ---- ---- ---- ---- ---- ---- ---- ---- ----\n");
	}

	_pZ80->SetContext(ctx);

	//
	//
	//
	
	switch (HIBYTE(wCode))
	{
		case 0:
			RAM[ctx.PC] = LOBYTE(wCode);
			break;

		case 1:
			RAM[ctx.PC] = 0xCB;
			RAM[ctx.PC + 1] = LOBYTE(wCode);
			break;

		case 2:
			RAM[ctx.PC] = 0xDD;
			RAM[ctx.PC + 1] = LOBYTE(wCode);
			break;

		case 3:
			RAM[ctx.PC] = 0xDD;
			RAM[ctx.PC + 1] = 0xCB;
			RAM[ctx.PC + 2] = 0x27; // arbitrary offset value
			RAM[ctx.PC + 3] = LOBYTE(wCode);
			break;

		case 4:
			RAM[ctx.PC] = 0xED;
			RAM[ctx.PC + 1] = LOBYTE(wCode);
			break;

		case 5:
			RAM[ctx.PC] = 0xFD;
			RAM[ctx.PC + 1] = LOBYTE(wCode);
			break;

		case 6:
			RAM[ctx.PC] = 0xFD;
			RAM[ctx.PC + 1] = 0xCB;
			RAM[ctx.PC + 2] = 0x27; // arbitrary offset value
			RAM[ctx.PC + 3] = LOBYTE(wCode);
			break;
		
		case 7:
			return 0;
			break;
	}

	WORD wCodePrev = wCode;
	
	wCode++;

	if (HIBYTE(wCodePrev) != HIBYTE(wCode))
		MessageBeep(0xFFFFFFFF);

	TCHAR szBuf[128];

	Z80_Dasm(&RAM[ctx.PC], szBuf, ctx.PC);

	TRACE("%04X %04X %04X %04X %04X %04X %04X %04X %04X %s\n",
		ctx.PC, ctx.AF, ctx.BC, ctx.DE, ctx.HL, ctx.IX, ctx.IY, ctx.SP, 0, szBuf);

	_pZ80->Exec(1);

	_pZ80->GetContext(ctx);

	Z80_Dasm(&RAM[ctx.PC], szBuf, ctx.PC);

	TRACE("%04X %04X %04X %04X %04X %04X %04X %04X %04X %s\n",
		ctx.PC, ctx.AF, ctx.BC, ctx.DE, ctx.HL, ctx.IX, ctx.IY, ctx.SP, 0, szBuf);

	_pZ80->Exec(1);

	_pZ80->GetContext(ctx);

	Z80_Dasm(&RAM[ctx.PC], szBuf, ctx.PC);

	TRACE("%04X %04X %04X %04X %04X %04X %04X %04X %04X %s\n",
		ctx.PC, ctx.AF, ctx.BC, ctx.DE, ctx.HL, ctx.IX, ctx.IY, ctx.SP, 0, szBuf);

	return 0;
}

static void Term()
{
	_pZ80->Release();
}

static void RenderScene(LPBYTE rgbBuffer, LONG lPitch)
{
}

GAME_DESCRIPTOR _gmdTestZ80 = 
{
	TEXT("Z80 Testbed"),
	NULL,
	Init,
	Exec,
	RenderScene,
	UpdateSound,
	Term
};

