/*---------------------------------------------------------------------------

	File:
		ay8910.cpp
	
	Abstract:
		Emulates the AY-3-8910 (and compatible) family of sound chips.

	Revisions:

	Copyright (C) 1997 Edward Massey

	References:  sound.c, by Ville Hallik, for MSX emulator; Mike Cuddy for
		psg.c module in MAME

---------------------------------------------------------------------------*/

#include "pch.h"
#include "misc.h"
#include "ay8910.h"

const BYTE AY8910::_rgbEnvelopes[16][32] =
{
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	  15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0 },
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 },
	{ 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0,
	  15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	  15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	  15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0 },
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }
};

#define AYClockFreq 1789750000
#define AYSoundRate 11025

AY8910_BASE::AY8910_BASE()
{
	Reset();
}

static void WritePort(int)
{
}

static int ReadPort()
{
	return 0;
}

void AY8910_BASE::Reset()
{
	ZERO_INIT(m_rgbRegs);

	m_rgbRegs[AY_ENABLE] = 077;
	m_NoiseGen = 1;
	m_Envelope = 15;
	m_StateNoise = 0;
	m_Incr0= m_Incr1 = m_Incr2 = m_Increnv = m_Incrnoise = 0;
//	m_latch = 0;

	m_pfnReadPortA = ReadPort;
	m_pfnWritePortA = WritePort;
	m_pfnReadPortB = ReadPort;
	m_pfnWritePortB = WritePort;
}

void AY8910_BASE::WriteReg(int r, int v)
{
	m_rgbRegs[r] = v;

	switch( r )
	{
		case AY_AVOL:
		case AY_BVOL:
		case AY_CVOL:
			m_rgbRegs[r] &= 0x1F;	/* mask volume */
			break;
		
		case AY_EFINE:
		case AY_ECOARSE:
			m_Countenv=0;
			/* fall through */
		
		case AY_ESHAPE:
			m_rgbRegs[AY_ESHAPE] &= 0xF;
			break;

		case AY_PORTA:
			m_pfnWritePortA(v);
			break;

		case AY_PORTB:
			m_pfnWritePortB(v);
			break;
	}

	int x;

	x = (m_rgbRegs[AY_AFINE]+((unsigned)(m_rgbRegs[AY_ACOARSE]&0xF)<<8));
	m_Incr0 = x ? AYClockFreq / AYSoundRate * 4 / x : 0;
	
	x = (m_rgbRegs[AY_BFINE]+((unsigned)(m_rgbRegs[AY_BCOARSE]&0xF)<<8));
	m_Incr1 = x ? AYClockFreq / AYSoundRate * 4 / x : 0;

	x = (m_rgbRegs[AY_CFINE]+((unsigned)(m_rgbRegs[AY_CCOARSE]&0xF)<<8));
	m_Incr2 = x ? AYClockFreq / AYSoundRate * 4 / x : 0;

	x = m_rgbRegs[AY_NOISEPER]&0x1F;
	m_Incrnoise = x ? AYClockFreq / AYSoundRate * 4 / x : 0;

	int cbBuffer = 512; // hack!!!

	x = (m_rgbRegs[AY_EFINE]+((unsigned)m_rgbRegs[AY_ECOARSE]<<8));
	m_Increnv = x ? AYClockFreq / AYSoundRate * 4 / x * cbBuffer : 0;

	m_Envelope = _rgbEnvelopes[m_rgbRegs[AY_ESHAPE]][(m_Countenv>>16)&0x1F];

	if( ( m_Countenv += m_Increnv ) & 0xFFE00000 )
	{
		switch( m_rgbRegs[AY_ESHAPE] )
		{
			case 8:
			case 10:
			case 12:
			case 14:
				m_Countenv -= 0x200000;
				break;

			default:
				m_Countenv = 0x100000;
				m_Increnv = 0;
		}
	}

	int Vol0 = ( m_rgbRegs[AY_AVOL] < 16 ) ? m_rgbRegs[AY_AVOL] : m_Envelope;
	int Vol1 = ( m_rgbRegs[AY_BVOL] < 16 ) ? m_rgbRegs[AY_BVOL] : m_Envelope;
	int Vol2 = ( m_rgbRegs[AY_CVOL] < 16 ) ? m_rgbRegs[AY_CVOL] : m_Envelope;

	/* if the frequency is 0, shut down the voice */
	if (m_Incr0 == 0)
		m_Vol0 = 0;
	else
		m_Vol0 = ( m_rgbRegs[AY_ENABLE] & 001 ) ? 0 : Vol0;

	if (m_Incr1 == 0)
		m_Vol1 = 0;
	else
		m_Vol1 = ( m_rgbRegs[AY_ENABLE] & 002 ) ? 0 : Vol1;

	if (m_Incr2 == 0)
		m_Vol2 = 0;
	else
		m_Vol2 = ( m_rgbRegs[AY_ENABLE] & 004 ) ? 0 : Vol2;

	if (m_Incrnoise == 0)
		m_Volnoise = 0;
	else
	{
		m_Volnoise = (
			((m_rgbRegs[AY_ENABLE] & 010) ? 0 : Vol0) +
			((m_rgbRegs[AY_ENABLE] & 020) ? 0 : Vol1) +
			((m_rgbRegs[AY_ENABLE] & 040) ? 0 : Vol2)) / 3;
	}
}

BYTE AY8910_BASE::ReadReg(int r)
{
	switch( r )
	{
		case AY_PORTA:
			return m_pfnReadPortA();

		case AY_PORTB:
			return m_pfnReadPortB();
	}

	return m_rgbRegs[r];
}

void AY8910_BASE::UpdateSound(LPBYTE rgbBuffer, LONG cbBuffer)
{
	int c0, c1, l0, l1, l2;

	while (cbBuffer--)
	{
		/*
		** These strange tricks are needed for getting rid
		** of nasty interferences between sampling frequency
		** and "rectangular sound" (which is also the output
		** of real AY-3-8910) we produce.
		*/

		c0 = m_Counter0;
		c1 = m_Counter0 + m_Incr0;
		l0 = ((c0&0x8000)?-16:16);

		if( (c0^c1)&0x8000 )
		{
			l0=l0*(0x8000-(c0&0x7FFF)-(c1&0x7FFF))/m_Incr0;
		}

		m_Counter0 = c1 & 0xFFFF;

		c0 = m_Counter1;
		c1 = m_Counter1 + m_Incr1;
		l1 = ((c0&0x8000)?-16:16);

		if( (c0^c1)&0x8000 )
		{
			l1=l1*(0x8000-(c0&0x7FFF)-(c1&0x7FFF))/m_Incr1;
		}

		m_Counter1 = c1 & 0xFFFF;

		c0 = m_Counter2;
		c1 = m_Counter2 + m_Incr2;
		l2 = ((c0&0x8000)?-16:16);
		
		if( (c0^c1)&0x8000 )
		{
			l2=l2*(0x8000-(c0&0x7FFF)-(c1&0x7FFF))/m_Incr2;
		}
		
		m_Counter2 = c1 & 0xFFFF;

		m_Countnoise &= 0xFFFF;

		if( ( m_Countnoise += m_Incrnoise ) & 0xFFFF0000 )
		{
			/*
			** The following code is a random bit generator :)
			*/
			m_StateNoise =
			( ( m_NoiseGen <<= 1 ) & 0x80000000
			? m_NoiseGen ^= 0x00040001 : m_NoiseGen ) & 1;
		}

		*rgbBuffer++ = AUDIO_CONV (
		( l0 * m_Vol0 + l1 * m_Vol1 + l2 * m_Vol2 ) / 6 +
		( m_StateNoise ? m_Volnoise : -m_Volnoise ) );
	}
}

void AY8910_BASE::UpdateSound(LPINT rgbBuffer, LONG cbBuffer)
{
	int c0, c1, l0, l1, l2;

	while (cbBuffer--)
	{
		/*
		** These strange tricks are needed for getting rid
		** of nasty interferences between sampling frequency
		** and "rectangular sound" (which is also the output
		** of real AY-3-8910) we produce.
		*/

		c0 = m_Counter0;
		c1 = m_Counter0 + m_Incr0;
		l0 = ((c0&0x8000)?-16:16);

		if( (c0^c1)&0x8000 )
		{
			l0=l0*(0x8000-(c0&0x7FFF)-(c1&0x7FFF))/m_Incr0;
		}

		m_Counter0 = c1 & 0xFFFF;

		c0 = m_Counter1;
		c1 = m_Counter1 + m_Incr1;
		l1 = ((c0&0x8000)?-16:16);

		if( (c0^c1)&0x8000 )
		{
			l1=l1*(0x8000-(c0&0x7FFF)-(c1&0x7FFF))/m_Incr1;
		}

		m_Counter1 = c1 & 0xFFFF;

		c0 = m_Counter2;
		c1 = m_Counter2 + m_Incr2;
		l2 = ((c0&0x8000)?-16:16);
		
		if( (c0^c1)&0x8000 )
		{
			l2=l2*(0x8000-(c0&0x7FFF)-(c1&0x7FFF))/m_Incr2;
		}
		
		m_Counter2 = c1 & 0xFFFF;

		m_Countnoise &= 0xFFFF;

		if( ( m_Countnoise += m_Incrnoise ) & 0xFFFF0000 )
		{
			/*
			** The following code is a random bit generator :)
			*/
			m_StateNoise =
			( ( m_NoiseGen <<= 1 ) & 0x80000000
			? m_NoiseGen ^= 0x00040001 : m_NoiseGen ) & 1;
		}

		*rgbBuffer++ += ( l0 * m_Vol0 + l1 * m_Vol1 + l2 * m_Vol2 ) / 6 +
		( m_StateNoise ? m_Volnoise : -m_Volnoise );
	}
}

