/*---------------------------------------------------------------------------

	File:
		dxutil.cpp

	Abstract:
		DirectX utilities

	Revisions:
		04-Feb-97 EAM Created

	Copyright (C) 1997 Edward Massey

---------------------------------------------------------------------------*/

#include "pch.h"
#include "dxutil.h"
#include "misc.h"
#include "resource.h"

struct EDM_DATA
{
	HWND hwndLb;
};

static HRESULT CALLBACK EnumDisplayModesProc(LPDDSURFACEDESC psd, LPVOID pContext)
{
	TStr<64> strLabel("%ix%ix%i",  psd->dwWidth, psd->dwHeight,
		psd->ddpfPixelFormat.dwRGBBitCount);

	const EDM_DATA &data = *(EDM_DATA*)pContext;

	int iItem = SendMessage(data.hwndLb, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)strLabel);

	if (iItem != -1)
	{
		DWORD dwMode = psd->dwWidth | (psd->dwHeight << 12) |
			(psd->ddpfPixelFormat.dwRGBBitCount << 24);
		
		SendMessage(data.hwndLb, LB_SETITEMDATA, iItem, dwMode);
	}

	return DDENUMRET_OK;
}

struct CDM_DATA
{
	LPDIRECTDRAW lpdd;
	LPDDSURFACEDESC pddsdInit;
	LPDDSURFACEDESC pddsdOut;
};

static BOOL CALLBACK ChooseDisplayModeDlgProc(HWND hwnd, UINT uMsg,
	WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:
		{
			CDM_DATA *pdata = (CDM_DATA*)lParam;

			EDM_DATA ctx;

			ctx.hwndLb = GetDlgItem(hwnd, IDC_LIST1);

			pdata->lpdd->EnumDisplayModes(0, pdata->pddsdInit, &ctx,
				EnumDisplayModesProc);

			return TRUE;
		}

		case WM_COMMAND:
		{
			switch (LOWORD(wParam))
			{
				case IDOK:
				{
					EndDialog(hwnd, IDOK);
					return TRUE;
				}

				case IDCANCEL:
					EndDialog(hwnd, IDCANCEL);
					return TRUE;
			}
		}
	}

	return FALSE;
}

HRESULT ChooseDisplayMode(HWND hwndOwner, LPDIRECTDRAW lpdd,
	LPDDSURFACEDESC pddsdInit, DDSURFACEDESC &ddsdOut)
{
	CDM_DATA data;

	data.pddsdInit = pddsdInit;
	data.pddsdOut  = &ddsdOut;
	data.lpdd = lpdd;

	int n = DialogBoxParam(GetModuleHandle(NULL),
		MAKEINTRESOURCE(IDD_CHOOSE_DISPLAY_MODE), hwndOwner,
		(DLGPROC)ChooseDisplayModeDlgProc, (LPARAM)&data);

	if (n == IDCANCEL)
		return E_ABORT;

	if (n == IDOK)
		return S_OK;

	return E_UNEXPECTED;
}

