/*---------------------------------------------------------------------------

	File:
		winmain.cpp

	Abstract:
		Main emulator file

	Revisions:
		17-Jan-97 EAM Created
		31-Jan-97 EAM Incorporated dynamic File menu modifications provided
			by Ivan Mackintosh.  Begin work on DirectX support.

	~95 fps w/o owndc, no graphics rendering into bmp
	~105 w/owndc
	~ fps w/o owndc, no graphics rendering into bmp, blitted @ 2X size
	250 - no rendering or blitting in release build, no throttling (equivilant to 10MHz Z80)
	67 - full rendering w/o blitting

	Build Options:
		Use identity palette
		Use private DC
		Use video for windows

	sprite buffer size will be...
	64 sprites * (16x16 pixels/sprite) * (1 byte/pixel) = 16KB
	... * (4 directions) = 64K
	... * (8 palettes) = 512K (yuch!)

	I'm working on a web site right now that explains everything about this
	emulator.  There's currently not much there, but keep checking back at:

	http://www.greenvillenc.com/home/edward/magex.htm

	Drawing modes...
	- windowed, fullscreen, screen mode (fullscreen only)
	- 0x0, 1x1, 2x2, fit window
	- display fps

	steps:
	- allow game to specify its buffer size, src rect, and palette
	- allow per-addr specification of callback for read/write in game
	- move appropriate globals to CEmWindow methods
	- sync this window mgr w/Hidra code
	- create a keystate array - set keystate to 1 on wm_keydown, to 0 on wm_keyup,
	  reset all to 0 on app deactivate
	- modify Z80 to use rgpfnMemRead[0x10000], _rgpfnMemWrite[0x10000];

	TODO:  clean up this messy code!!!!!
	TODO:  use two or more audio buffers
	TODO:  grab new image library (image.cpp, image.h) from demo
	TODO:  make a DirectX helper module
	TODO:  make an audio module and put audio stuff from here there

	Code clean-up...
	 * localize sound driver code and utility routines to a separate module
	 * localize DirectDraw

	Copyright (C) 1997 Edward Massey

---------------------------------------------------------------------------*/

#include "pch.h"
#include "../inc/game.h"
#include "emwindow.h"
#include "misc.h"
#include "resource.h"
#include "dxutil.h"
#include "script.h"

extern SCRIPT_GAME _rgGames[];
extern int _cGames;

extern HRESULT RegInit();

extern const GAME_DESCRIPTOR *_rgpGameDesc[];

extern "C" void TRACE(LPCTSTR,...);

// consider: use a global _pActiveGame to avoid the indexing each time
extern const GAME_DESCRIPTOR *_pgmdActive;

GAME_DESCRIPTOR _gmdActive;

static inline const GAME_DESCRIPTOR *GetActiveGame()
{
	return _pgmdActive;
//	return _rgpGameDesc[_nCurrentGame];
}

#define TITLE TEXT("MageX")

#define ID_GAMEBASE     (32600)
#define ID_GAMEMAX      (32699)
#define ID_MODEBASE     (32700)
#define ID_MODEMAX      (32799)

static const GUID CLSID_DirectDraw = { 0xD7B70EE0,0x4340,0x11CF,0xB0,0x63,0x00,0x20,0xAF,0xC2,0xCD,0x35 };
static const GUID IID_IDirectDraw  = { 0x6C14DB80,0xA733,0x11CE,0xA5,0x21,0x00,0x20,0xAF,0x0B,0xE5,0x60 };
static const GUID IID_IDirectDraw2 = { 0xB3A6F3E0,0x2B43,0x11CF,0xA2,0xDE,0x00,0xAA,0x00,0xB9,0x33,0x56 };

static const GUID CLSID_DirectSound      = { 0x47d4d946,0x62e8,0x11cf,0x93,0xbc,0x44,0x45,0x53,0x54,0x0, 0x0 };
static const GUID IID_IDirectSound       = { 0x279AFA83,0x4981,0x11CE,0xA5,0x21,0x00,0x20,0xAF,0x0B,0xE5,0x60};
static const GUID IID_IDirectSoundBuffer = { 0x279AFA85,0x4981,0x11CE,0xA5,0x21,0x00,0x20,0xAF,0x0B,0xE5,0x60};

extern "C" {
	unsigned char RAM[0x10000];   /* 64k of RAM */
	extern int _cInterrupts, _cInstructions, _cTStates;
};

int _fThrottle = TRUE;
BOOL _fJoystickEnable = FALSE;
BOOL _fCpuStats = FALSE;

static unsigned long cFrame = 0;
static DWORD dwTick;
int _cFps;

// most of these should be CEmWindow members...
static HPALETTE _hpal;
static HBITMAP _hbmp;
static LPBYTE _pvBits;
static BITMAPINFO *_pbmi;
static BITMAP _bmp;

// sound state...
BOOL _fSoundActive = FALSE;

static HDC _hdc;

static LPDIRECTDRAWSURFACE _pddsPrimary;   // DirectDraw primary surface
static LPDIRECTDRAWSURFACE _pddsBack;      // DirectDraw back surface
static RECT _rcDest = { 0, 0, 256, 256 };
static RECT _rcSrc = { 0, 0, 256, 256 }; // TODO: allow game to specify this
static DDSURFACEDESC _ddsd;
static LPDIRECTDRAWCLIPPER _pddClipper; // should be window member? m_pClipper
static LPDIRECTDRAWPALETTE _pddPalette;

static int _nBlitMode = 1;

static BOOL _fQuitProgram = FALSE;
static int _nCurrentGame = 0;
void BuildFileMenu(HWND hwnd);
static const GAME_DESCRIPTOR *_pgmdActive;

struct GLOBAL
{
	BOOL fAppActive;
	BOOL fAppPaused;
	BOOL fUseSystemMemory;
	BOOL fUseVideoMemory;
	BOOL fDirectXInstalled;
	BOOL fUseDirectX;
};

GLOBAL _glb = {
	TRUE,
	FALSE,
	TRUE,
	FALSE,
	FALSE,
	TRUE
};

BOOL WndCenter(HWND hwndChild, HWND hwndParent);

static inline void DoBlit()
{
	// NOTE:  This code isn't used when running full screen.  We can use
	//        this technique (StretchDIBits into the back buffer) so we can
	//        switch in and out of full screen mode.
	
	if (_pddsPrimary != NULL)
	{
/*
		HDC hdc;

		if (SUCCEEDED(_pddsBack->GetDC(&hdc)))
		{
			StretchDIBits(hdc, 0, 0, _bmp.bmWidth, _bmp.bmHeight,
				0, 0, _bmp.bmWidth, _bmp.bmHeight,
				_bmp.bmBits, _pbmi, DIB_PAL_COLORS, SRCCOPY);

			_pddsBack->ReleaseDC(hdc);
		}

		//
		//
		//

		hr = _pddsPrimary->Blt(&_rcDest, _pddsBack, &_rcSrc, 0, NULL );

		if (hr == DDERR_SURFACELOST)
		{
			if (SUCCEEDED(_pddsPrimary->Restore()) &&
				SUCCEEDED(_pddsBack->Restore()))
			{
			}
		}

#ifdef _DEBUG
		if (FAILED(hr))
			MessageBeep((UINT)-1);
#endif

*/
/*
		HDC hdc;

//		if (SUCCEEDED(_pddsBack->GetDC(&hdc)))
		if (SUCCEEDED(hr = _pddsPrimary->GetDC(&hdc)))
		{
//			MessageBeep(-1);

			StretchDIBits(hdc, 0, 0, _bmp.bmWidth, _bmp.bmHeight,
				0, 0, _bmp.bmWidth, _bmp.bmHeight,
				_bmp.bmBits, _pbmi, DIB_PAL_COLORS, SRCCOPY);

			_pddsPrimary->ReleaseDC(hdc);
//			_pddsBack->ReleaseDC(hdc);
		}

//		hr = _pddsPrimary->Blt(&_rcDest, _pddsBack, &_rcSrc, 0, NULL );

		if (hr == DDERR_SURFACELOST)
		{
			if (SUCCEEDED(_pddsPrimary->Restore()) &&
				SUCCEEDED(_pddsBack->Restore()))
			{
			}
		}
*/
	}
	else
	{
		// TODO: eliminate this decision here!  store the effective bmp width and height
		switch (_nBlitMode)
		{
			case 1:
			{
				StretchDIBits(_hdc, -_rcSrc.left, -_rcSrc.top, _bmp.bmWidth, _bmp.bmHeight,
					0, 0, _bmp.bmWidth, _bmp.bmHeight,
					_bmp.bmBits, _pbmi, DIB_PAL_COLORS, SRCCOPY);
				
				break;
			}

			case 2:
			{
				StretchDIBits(_hdc, -_rcSrc.left, -_rcSrc.top, _bmp.bmWidth * 2, _bmp.bmHeight * 2,
					0, 0, _bmp.bmWidth, _bmp.bmHeight,
					_bmp.bmBits, _pbmi, DIB_PAL_COLORS, SRCCOPY);
				
				break;
			}
		}
	}
}

HWND hwndMain;

BOOL IdleShowMem()
{
	int X, Y;

	if (_pvBits != NULL)
	{
		for (Y = 0; Y < 256; Y++)
		{
			for (X = 0; X < 256; X++)
			{
				_pvBits[_bmp.bmWidthBytes * Y + X] = RAM[256 * Y + X] >> 4;
			}
		}

		DoBlit();
	}

	return TRUE;
}

BOOL IdleCharset()
{
//	(void)GetActiveGame->pfnDrawCharset();

	int X, Y;

	for (Y = 0; Y < 16; Y++)
	{
		for (X = 0; X < 16; X++)
		{
//			DrawChar(X, Y, Y * 16 + X, 0);
		}
	}

	DoBlit();

	return TRUE;
}

static LPDIRECTDRAW _lpdd = NULL;
static BOOL _fWaitVBlank = FALSE;

static int _cFrame;
static int _cFrameMod = 2;

extern void SoundDraw(LPBYTE rgbBuffer, LONG lPitch);

// TODO: add frameskip options for really slow computers

BOOL IdleNormal()
{
/*
	_cFrame++;

	if ((_cFrame++ % _cFrameMod) == 0)
		return TRUE;
*/

	HRESULT hr;

	if (_pddsPrimary != NULL && _pddsBack != NULL)
	{
		DDSURFACEDESC ddsd;

		ddsd.dwSize = sizeof(ddsd);

		hr = _pddsBack->Lock(NULL, &ddsd, DDLOCK_WAIT | DDLOCK_WRITEONLY, NULL);

		if (SUCCEEDED(hr))
		{
			GetActiveGame()->pfnDrawFrame((LPBYTE)ddsd.lpSurface, ddsd.lPitch);

			_pddsBack->Unlock(ddsd.lpSurface);

			//
			//  TODO:  Move this calculation out of the loop!
			//  TODO:  Clip the rects if needed
			//  TODO:  align rects on 32-bit boundaries?
			//  TODO:  Use BltFast!!!
			//  TODO:  allow use of pgup/pgdown to scroll if screen too small
			//

			RECT rcSrc = _rcSrc;

			const SIZE siz = {
				(_rcSrc.right - _rcSrc.left) * _nBlitMode,
				(_rcSrc.bottom - _rcSrc.top) * _nBlitMode
			};

			RECT rcDest;

			if (siz.cx > _ddsd.dwWidth)
			{
				rcDest.left = 0;
				rcDest.right = _ddsd.dwWidth;
				rcSrc.left = (siz.cx - _ddsd.dwWidth) / 2;
				rcSrc.right = rcSrc.left + _ddsd.dwWidth;
			}
			else
			{
				rcDest.left = (_ddsd.dwWidth - siz.cx) / 2;
				rcDest.right = rcDest.left + siz.cx;
			}

			if (siz.cy > _ddsd.dwHeight)
			{
				rcDest.top = 0;
				rcDest.bottom = _ddsd.dwHeight;
				rcSrc.top = (siz.cy - _ddsd.dwHeight) / 2;
				rcSrc.bottom = rcSrc.top + _ddsd.dwHeight;
			}
			else
			{
				rcDest.top = (_ddsd.dwHeight - siz.cy) / 2;
				rcDest.bottom = rcDest.top + siz.cy;
			}
			
//			rcDest.left = 0;
//			rcDest.top = 0;
//			rcDest.right = _ddsd.dwWidth;
//			rcDest.bottom = _ddsd.dwHeight;

			if (_fWaitVBlank)
				_lpdd->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, NULL);

			hr = _pddsPrimary->Blt(&rcDest, _pddsBack, &rcSrc, DDBLT_WAIT, NULL);
//			hr = _pddsPrimary->BltFast(0, 0, _pddsBack, &_rcSrc, DDBLTFAST_NOCOLORKEY | DDBLT_WAIT);

			if (hr == DDERR_SURFACELOST)
				_pddsPrimary->Restore();
		}
		else
		{
			if (hr == DDERR_SURFACELOST)
				_pddsBack->Restore();
		}

#if 0 // def _DEBUG
		if (FAILED(hr))
			MessageBeep((UINT)-1);
#endif
	}
	else
	{
		(void)GetActiveGame()->pfnDrawFrame(_pvBits, _bmp.bmWidthBytes);

		SoundDraw(_pvBits, _bmp.bmWidthBytes);

		DoBlit();
	}

	return TRUE;
}

BOOL IdleBlitOnly()
{
	HRESULT hr;

	if (_pddsPrimary != NULL && _pddsBack != NULL)
	{
		DDSURFACEDESC ddsd;

		ddsd.dwSize = sizeof(ddsd);

		hr = _pddsBack->Lock(NULL, &ddsd, DDLOCK_WAIT | DDLOCK_WRITEONLY, NULL);

		if (SUCCEEDED(hr))
		{
			_pddsBack->Unlock(ddsd.lpSurface);

			if (SUCCEEDED(hr))
			{
				//
				//  TODO:  Move this calculation out of the loop!
				//  TODO:  Clip the rects if needed
				//  TODO:  align rects on 32-bit boundaries?
				//

				RECT rcSrc = _rcSrc;

				const SIZE siz = {
					(_rcSrc.right - _rcSrc.left) * _nBlitMode,
					(_rcSrc.bottom - _rcSrc.top) * _nBlitMode
				};

				RECT rcDest;

				if (siz.cx > _ddsd.dwWidth)
				{
					rcDest.left = 0;
					rcDest.right = _ddsd.dwWidth;
					rcSrc.left = (siz.cx - _ddsd.dwWidth) / 2;
					rcSrc.right = rcSrc.left + _ddsd.dwWidth;
				}
				else
				{
					rcDest.left = (_ddsd.dwWidth - siz.cx) / 2;
					rcDest.right = rcDest.left + siz.cx;
				}

				if (siz.cy > _ddsd.dwHeight)
				{
					rcDest.top = 0;
					rcDest.bottom = _ddsd.dwHeight;
					rcSrc.top = (siz.cy - _ddsd.dwHeight) / 2;
					rcSrc.bottom = rcSrc.top + _ddsd.dwHeight;
				}
				else
				{
					rcDest.top = (_ddsd.dwHeight - siz.cy) / 2;
					rcDest.bottom = rcDest.top + siz.cy;
				}
				
				hr = _pddsPrimary->Blt(&rcDest, _pddsBack, &rcSrc, DDBLT_WAIT, NULL);
//				hr = _pddsPrimary->BltFast(0, 0, _pddsBack, &_rcSrc, DDBLTFAST_NOCOLORKEY | DDBLT_WAIT);

				if (hr == DDERR_SURFACELOST)
					_pddsPrimary->Restore();
			}
		}
		else
		{
			if (hr == DDERR_SURFACELOST)
				_pddsBack->Restore();
		}
	}
	else
	{
		DoBlit();
	}

	return TRUE;
}

BOOL IdleNop()
{
	return TRUE;
}

typedef BOOL (*IDLEPROC)();

static IDLEPROC _pfnIdle = IdleNormal;

static BOOL inline DoIdle()
{
	cFrame++;

	return _pfnIdle();
}

HRESULT CreateObjs(LPDIRECTDRAW lpdd, PALETTEENTRY *rgpe, HWND hwnd,
	const GAME_LOAD_INFO &gli)
{
	if (lpdd == NULL)
		return E_FAIL;

	HRESULT hr;

	//
	//
	//

	DDCAPS ddcHAL, ddcHEL;

	ddcHAL.dwSize = sizeof(ddcHAL);
	ddcHEL.dwSize = sizeof(ddcHEL);

	lpdd->GetCaps(&ddcHAL, &ddcHEL);

	//
	//  Clean up DirectX objects
	//

	if (_pddPalette != NULL)
		_pddPalette->Release();

	if (_pddClipper != NULL)
		_pddClipper->Release();

	if (_pddsPrimary != NULL)
		_pddsPrimary->Release();

	if (_pddsBack != NULL)
		_pddsBack->Release();

	DDSURFACEDESC ddsd;

	ZERO_INIT(ddsd);

	ddsd.dwSize = sizeof(ddsd);

	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

	hr = lpdd->CreateSurface( &ddsd, &_pddsPrimary, NULL );
	
	if (SUCCEEDED(hr))
	{
		//
		//  Setup clipper (not actually used since we're always full screen)
		//
		
		if (SUCCEEDED(lpdd->CreateClipper(0, &_pddClipper, NULL)))
		{
			_pddClipper->SetHWnd(0, hwnd);

			_pddsPrimary->SetClipper(_pddClipper);
		}

		//
		//  Setup palette
		//

		if (SUCCEEDED(lpdd->CreatePalette(DDPCAPS_8BIT, rgpe, &_pddPalette, NULL)))
		{
			_pddsPrimary->SetPalette(_pddPalette);
//						_pddsBack->SetPalette(_pddPalette);
		}

		//
		//  Create surface
		//
		
		ZERO_INIT(ddsd);
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;

		if (_glb.fUseSystemMemory)
			ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
		else if (_glb.fUseVideoMemory)
			ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
		else
			ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;

		ddsd.dwWidth = gli.sizBuffer.cx;
		ddsd.dwHeight = gli.sizBuffer.cy;

		hr = lpdd->CreateSurface(&ddsd, &_pddsBack, NULL);
	}

	return hr;
}

HRESULT ChooseDisplayMode(HWND hwndOwner, LPDIRECTDRAW lpdd,
	LPDDSURFACEDESC pddsdInit, DDSURFACEDESC &ddsdOut);

GAME_LOAD_INFO _gli;
PALETTEENTRY _rgpe[256];

static int CheckMessages();
extern void Throttle(int);

void PlayBall(HWND hwnd, LPDIRECTDRAW lpdd)
{
	TCHAR szMsg[256] = "";
	PALETTEENTRY *rgpe = _rgpe; // [256];

	GAME_LOAD_INFO &gli = _gli;

	ZERO_INIT(gli);

	gli.ppe = rgpe;
	gli.pszMsg = szMsg;
	gli.cchMsgMax = DIM(szMsg);
	gli.pvContext = GetActiveGame()->pvContext;

	ZERO_INIT(RAM);
	
	if (FAILED(GetActiveGame()->pfnLoad(gli)))
	{
		MessageBox(hwnd, TStr<256>("%s: %s", GetActiveGame()->pszTitle, szMsg),
			TEXT("Unable to load game"), MB_ICONERROR | MB_OK);

		static BOOL _fDisplayFullBuffer = FALSE;
		
		if (_fDisplayFullBuffer)
		{
			gli.rcDisplay.left   = 0;
			gli.rcDisplay.top    = 0;
			gli.rcDisplay.right  = gli.sizBuffer.cx;
			gli.rcDisplay.bottom = gli.sizBuffer.cy;
		}

		_fQuitProgram = TRUE; // HACK

		return;

		//
		//
		//

		MSG msg;

		while (GetMessage(&msg, NULL, 0, 0))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	else
	{
		if (lpdd != NULL)
		{
			CreateObjs(lpdd, rgpe, hwnd, gli);
		}
		else
		{
			//
			//
			//

			RECT rcClient = { 0, 0, gli.rcDisplay.right - gli.rcDisplay.left,
				gli.rcDisplay.bottom - gli.rcDisplay.top };
			
			if (AdjustWindowRect(&rcClient, GetWindowLong(hwnd, GWL_STYLE), TRUE))
			{
				SetWindowPos(hwnd, NULL, 0, 0, rcClient.right - rcClient.left,
					rcClient.bottom - rcClient.top, SWP_NOZORDER | SWP_NOMOVE);
			}
			
			//
			//
			//

			BYTE rgbPalette[sizeof(LOGPALETTE) + 255 * sizeof(PALETTEENTRY)];
			
			LOGPALETTE &Palette = *(LOGPALETTE*)rgbPalette;

			Palette.palVersion = 0x0300;
			Palette.palNumEntries = gli.cpe;

			for (UINT ipe = 0; ipe < gli.cpe; ipe++)
			{
				Palette.palPalEntry[ipe] = rgpe[ipe];
				Palette.palPalEntry[ipe].peFlags = PC_NOCOLLAPSE;
			}

			if (_hpal != NULL)
				DeleteObject(_hpal);

			_hpal = CreatePalette(&Palette);
//			_hpal = CreateIdentityPalette((RGBQUAD*)Palette.aEntries, DIM(Palette.aEntries));

			GetPaletteEntries(_hpal, 0, 32, rgpe);

			//
			//
			//

			SelectPalette(_hdc, _hpal, FALSE);
			RealizePalette(_hdc);

			if (_pbmi != NULL)
				free(_pbmi);
				
			_pbmi = (BITMAPINFO*)malloc(sizeof(BITMAPINFOHEADER) + gli.cpe * sizeof(RGBQUAD));

			if (_pbmi != NULL)
			{
				BITMAPINFO &bmi = *_pbmi;

				bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
				bmi.bmiHeader.biWidth = gli.sizBuffer.cx;
				bmi.bmiHeader.biHeight = -gli.sizBuffer.cy; // NOTE: top-down DIB
				bmi.bmiHeader.biPlanes = 1;
				bmi.bmiHeader.biBitCount = 8;
				bmi.bmiHeader.biCompression = BI_RGB;
				bmi.bmiHeader.biSizeImage = 0;
				bmi.bmiHeader.biXPelsPerMeter = 0;
				bmi.bmiHeader.biYPelsPerMeter = 0;
				bmi.bmiHeader.biClrUsed = gli.cpe;
				bmi.bmiHeader.biClrImportant = 0;

				for (UINT i = 0; i < gli.cpe; i++)
					((WORD*)(bmi.bmiColors))[i] = i;

				if (_hbmp != NULL)
					DeleteObject(_hbmp);

				_hbmp = CreateDIBSection(_hdc, &bmi, DIB_PAL_COLORS, (void**)&_pvBits, NULL, 0);

				if (_hbmp != NULL)
				{
					ZERO_INIT(_bmp);
					
					if (GetObject(_hbmp, sizeof(_bmp), &_bmp) > 0)
					{
					}
				}
			}
		}

		_rcSrc = gli.rcDisplay;

		//
		//  Call the game driver's run procedure and pass its opaque
		//  context information.
		//

/*
		_cInterrupts = 0;
		_cInstructions = 0;
		_cTStates = 0;
*/

		DWORD dwCurrTime = timeGetTime();

		const GAME_DESCRIPTOR *pgdTerm = GetActiveGame(); // HACK!!!

		for (;;)
		{
			GetActiveGame()->Exec();

			if (!CheckMessages())
				break;

			Throttle(60);
		}

		pgdTerm->pfnTerm();

/*
		TODO:  gather stats...  time in exec, time in render, time in blit, time in sound, time waiting (throttle), number of frames
		show as millisec and relative percent of frame time
		also show effective CPU speed (cycles/second)

		enum
		{
			GM_MENU,
			GM_GRAPHICS,
			GM_SOUNDS,
			GM_PAUSED,
			GM_RUNNING,
			GM_SETTINGS,
			GM_PALETTE
		};

		if (SUCCEEDED(hr = _gmdActive->Init(gli)))
		{
			if (SUCCEEDED(CreateObjs(gli)))
			{
				if (fStats)
				{
					DWORD rgdwTimes[] = { 0, 0, 0, 0, 0 };
					DWORD cCyclesTotal = 0;

					for (;;)
					{
						if (!CheckMessages())
							break;

						nThrottle: 0 - none, > 0 = fps, -1 = vsync
						Throttle(nThrottle);

						dwTimePre = ...
						int cCycles = _gmdActive->Exec();
						cCyclesTotal += cCycles;
						dwTimePost = ...

						dwTimeExec = ...
					}
				}
				else
				{
					for (;;)
					{
						if (KeyState(VK_ESCAPE))
						{
							point exec, video, audio function pointers to menu code
						}
						else if (KeyState('P'))
						{
						}

						int cCycles = _gmdActive->Exec();

						cCyclesTotal += cCycles;

						_pfnUpdateAudio(rgbBuffer, cbBuffer);

						_pfnUpdateVideo(rgbBuffer, lPitch);

						if (!CheckMessages())
							break;

						nThrottle: 0 - none, > 0 = fps, -1 = vsync
						Throttle(nThrottle);
					}
				}

				_gmdActive->Term();
			}
		}
*/

		DWORD dwTime = timeGetTime() - dwCurrTime;

		//
		//  TODO:  Add t-states per callback (set point and actual), copy stats to
		//         clipboard, calc t-states/instruction, show CPU name, game name,
		//         callbacks/sec, CPU Speed
		//

		if (_fCpuStats)
		{
/*
			MessageBox(hwnd, TStr<256>("_cInterrupts=%i, _cInstructions=%i, _cTStates=%i, %i milliseconds",
				_cInterrupts, _cInstructions, _cTStates, dwTime), "CPU Emulation Stats",
				MB_ICONINFORMATION);
*/
		}
	}
}

static SIZE _rgsizMode[100];
static int _iMode = -1;

static HRESULT CALLBACK EnumDisplayModesProc(LPDDSURFACEDESC psd, LPVOID pContext)
{
	if (psd->ddpfPixelFormat.dwRGBBitCount == 8)
	{
		TStr<64> strLabel("%ix%ix%i", psd->dwWidth, psd->dwHeight,
			psd->ddpfPixelFormat.dwRGBBitCount);

		HMENU hmenu = (HMENU)pContext;

		static int i = 0;

		_rgsizMode[i].cx = psd->dwWidth;
		_rgsizMode[i].cy = psd->dwHeight;

		MENUITEMINFO mii;

		mii.cbSize = sizeof(mii);

		if (i == 0)
		{
			mii.fMask = MIIM_TYPE;
			mii.fType = MFT_SEPARATOR;

			InsertMenuItem(hmenu, GetMenuItemCount(hmenu), TRUE, &mii);
		}

		mii.fType      = MFT_STRING;
		mii.fMask      = MIIM_TYPE | MIIM_ID | MIIM_DATA;
		mii.wID        = ID_MODEBASE + i;
		mii.dwTypeData = strLabel;
		mii.cch        = lstrlen(mii.dwTypeData);
		mii.dwItemData = psd->dwWidth | (psd->dwHeight << 12) |
			(psd->ddpfPixelFormat.dwRGBBitCount << 24);

		i = i + 1;

		InsertMenuItem(hmenu, GetMenuItemCount(hmenu), TRUE, &mii);
	}

	return DDENUMRET_OK;
}

HRESULT DxInit(LPDIRECTDRAW &lpdd)
{
	HRESULT hr;
	
	lpdd = NULL;

	hr = CoInitialize(NULL);
	
	if (FAILED(hr))
		return hr;
		
	hr = CoCreateInstance(CLSID_DirectDraw, NULL, CLSCTX_ALL,
		IID_IDirectDraw, (void**)&lpdd);
			
	if (SUCCEEDED(hr))
	{
		hr = lpdd->Initialize(NULL);
		
		if (FAILED(hr))
		{
			lpdd->Release();
			lpdd = NULL;
		}
	}

	return hr;
}

static BOOL CALLBACK LicenseDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static BOOL CALLBACK WelcomeDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

BOOL _fDisplayLicense = FALSE;

LPDIRECTSOUND _lpDirectSound = NULL;
LPDIRECTSOUNDBUFFER _rgpDirectSoundBuffer[1];

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow)
{
	ScriptInit("c:\\dev\\magex\\src\\main\\script3.txt");

	RegInit();

	//
	//  Display the license
	//

	if (_fDisplayLicense)
	{
		if (DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_LICENSE),
				NULL, (DLGPROC)LicenseDlgProc) != IDYES)
		{
			return 0;
		}
	}

	//
	//
	//

	HRESULT hr;
	int n;
	BOOL fCoInit = FALSE;
	_pgmdActive = _rgpGameDesc[_nCurrentGame];

	if (lstrlen(lpCmdLine) == 0)
		lpCmdLine = "dkong";

//	ScriptFindGame(lpCmdLine, game);

	LPDIRECTDRAW lpdd = NULL;

	//
	//  Use OLE to see if we can get a DirectX object
	//

	_glb.fDirectXInstalled = FALSE;

	if (SUCCEEDED(CoInitialize(NULL)))
	{
		fCoInit = TRUE;

		if (SUCCEEDED(CoCreateInstance(CLSID_DirectDraw, NULL, CLSCTX_ALL,
			IID_IDirectDraw, (void**)&lpdd)))
		{
			if (SUCCEEDED(lpdd->Initialize(NULL)))
			{
				_glb.fDirectXInstalled = TRUE;
			}
			else
			{
				lpdd->Release();
				lpdd = NULL;
			}
		}

/*
		if (GetAsyncKeyState(VK_SHIFT) < 0 && SUCCEEDED(CoCreateInstance(CLSID_DirectSound, NULL, CLSCTX_ALL,
			IID_IDirectSound, (void**)&_lpDirectSound)))
		{
			if (SUCCEEDED(_lpDirectSound->Initialize(NULL)))
			{
			}
			else
			{
				_lpDirectSound->Release();
				_lpDirectSound = NULL;
			}
		}
*/
	}

	n = DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_WELCOME),
		NULL, (DLGPROC)WelcomeDlgProc);

	if (n == IDOK)
	{
		if (_glb.fUseDirectX)
		{
			_ddsd.dwSize = sizeof(_ddsd);
			_ddsd.dwFlags = DDSD_PIXELFORMAT;
			_ddsd.ddpfPixelFormat.dwSize = sizeof(_ddsd.ddpfPixelFormat);
			_ddsd.ddpfPixelFormat.dwFlags = DDPF_RGB;
			_ddsd.ddpfPixelFormat.dwRGBBitCount = DDBD_8;
			_ddsd.dwWidth = 640;
			_ddsd.dwHeight = 480;
			_ddsd.ddpfPixelFormat.dwRGBBitCount = 8;
		}
		else
		{
			if (lpdd != NULL)
			{
				lpdd->Release();

				lpdd = NULL;
			}
		}
	}
	else
	{
		if (lpdd != NULL)
			lpdd->Release();
		
		CoUninitialize();
		
		ScriptTerm();

		return 1;
	}

	/////////////////////////////////////////////////////////////////////////

	//
	//
	//

	IWindow *pwnd = new CEmWindow;

	if (pwnd != NULL)
	{
		//
		//
		//

		LPCTSTR pszWndClass = WndRegisterWindowClass(
			TEXT("GameWindowClass"), CS_HREDRAW | CS_VREDRAW | CS_OWNDC);

		if (pszWndClass != NULL)
		{
			HWND hwnd = WndCreateWindow(0, pszWndClass, "",
//				WS_OVERLAPPEDWINDOW, 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), // _ddsd.dwWidth, _ddsd.dwHeight,
				WS_OVERLAPPEDWINDOW, 0, 0, 256, 256, // CW_USEDEFAULT, CW_USEDEFAULT, // _ddsd.dwWidth, _ddsd.dwHeight,
				NULL, LoadMenu(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_MAIN)),
				pwnd);

			if (hwnd != NULL)
			{
				hwndMain = hwnd;

				ShowWindow(hwnd, nCmdShow);

				if (_lpDirectSound != NULL)
				{
					hr = _lpDirectSound->SetCooperativeLevel(hwnd, DSSCL_EXCLUSIVE);

					if (SUCCEEDED(hr))
					{
						WAVEFORMATEX wf;
						
						wf.wFormatTag      = WAVE_FORMAT_PCM;
						wf.nChannels       = 1;
						wf.nSamplesPerSec  = 11025;
						wf.wBitsPerSample  = 8;
						wf.nBlockAlign     = (wf.wBitsPerSample / 8) * wf.nChannels;
						wf.nAvgBytesPerSec = wf.nSamplesPerSec * wf.nBlockAlign;
						wf.cbSize          = sizeof(wf);

						DSBUFFERDESC dsbd;

						dsbd.dwSize = sizeof(dsbd);
						dsbd.dwFlags = DSBCAPS_GETCURRENTPOSITION2;
						dsbd.dwBufferBytes = 256;
						dsbd.dwReserved = 0;
						dsbd.lpwfxFormat = &wf;

						hr = _lpDirectSound->CreateSoundBuffer(&dsbd, &_rgpDirectSoundBuffer[0], NULL);

						if (SUCCEEDED(hr))
						{
							_rgpDirectSoundBuffer[0]->SetFormat(&wf);
							_rgpDirectSoundBuffer[0]->Play(0, 0, DSBPLAY_LOOPING);
						}
					}
					else
					{
					}
				}

				if (lpdd != NULL)
				{
					hr = lpdd->SetCooperativeLevel(hwnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_ALLOWMODEX | DDSCL_ALLOWREBOOT);

					hr = lpdd->SetDisplayMode(_ddsd.dwWidth, _ddsd.dwHeight,
						_ddsd.ddpfPixelFormat.dwRGBBitCount);

					_lpdd = lpdd; // hack-o-matic so menu can change mode

					lpdd->EnumDisplayModes(0, NULL, GetSubMenu(GetMenu(hwnd), 2),
						EnumDisplayModesProc);
				}

				UpdateWindow(hwnd);

				//
				//
				//

				do
				{
					PlayBall(hwnd, lpdd);
				} while ((!_fQuitProgram));

				//
				//
				//

				DestroyWindow(hwnd);
			}
			else
			{
				n = FALSE;
			}

			UnregisterClass(pszWndClass, hInstance);
		}
		else
		{
			n = FALSE;
		}

		pwnd->Release();
	}
	else
	{
		n = FALSE;
	}

	//
	//  Clean up DirectX objects and COM library
	//

	if (_pddPalette != NULL)
		_pddPalette->Release();

	if (_pddClipper != NULL)
		_pddClipper->Release();

	if (_pddsPrimary != NULL)
		_pddsPrimary->Release();

	if (_pddsBack != NULL)
		_pddsBack->Release();

	if (lpdd != NULL)
		lpdd->Release();

	if (_rgpDirectSoundBuffer[0] != NULL)
	{
		_rgpDirectSoundBuffer[0]->Stop();
		_rgpDirectSoundBuffer[0]->Release();
	}

	if (_lpDirectSound != NULL)
		_lpDirectSound->Release();

	if (fCoInit)
		CoUninitialize();

	ScriptTerm();

	return n;
}

// called from Interrupt

int _nPause;
#include "../inc/osdepend.h"

extern void SoundPump(void (*pfnUpdateSound)(LPBYTE rgbBuffer, LONG cbBuffer));

static int CheckMessages()
{
	//
	//	Handle any pending messages
	//
	
	MSG msg;

	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if (msg.message == WM_QUIT)
		    return FALSE;
		
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	//
	//
	//
	
	if (_fSoundActive && _pgmdActive->pfnUpdateSound != NULL)
	{
		if (_rgpDirectSoundBuffer[0] != NULL)
		{
			DWORD cbWrite = 256;
			
			LPVOID pvData1, pvData2;
			DWORD cbData1, cbData2;

//			DWORD dwCurrentPlayCursor;
			DWORD dwCurrentWriteCursor;
				
//			_rgpDirectSoundBuffer[0]->GetCurrentPosition(&dwCurrentPlayCursor,
//				&dwCurrentWriteCursor);

			dwCurrentWriteCursor = 0;

//			static DWORD dwPrevPlayCursor;

//			if (dwPrevPlayCursor + 256 < dwCurrentPlayCursor)
			{
//				dwPrevPlayCursor = dwCurrentPlayCursor;
				
				if (SUCCEEDED(_rgpDirectSoundBuffer[0]->Lock(dwCurrentWriteCursor,
					cbWrite, &pvData1, &cbData1, &pvData2, &cbData2, 0)))
				{
					_pgmdActive->pfnUpdateSound((LPBYTE)pvData1, cbData1);

					if (pvData2 != NULL)
						_pgmdActive->pfnUpdateSound((LPBYTE)pvData2, cbData2);

					_rgpDirectSoundBuffer[0]->Unlock(pvData1, cbData1, pvData2, cbData2);
				}
			}
		}
		else
		{
			SoundPump(_pgmdActive->pfnUpdateSound);
		}
	}

	//
	//
	//
	
	DoIdle();
	
	if (_nPause != 0)
		WaitMessage();

	return TRUE;
}

CEmWindow::CEmWindow()
{
	m_cRef = 1;
}

CEmWindow::~CEmWindow()
{
}

HRESULT CEmWindow::QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject)
{
	if (riid == IID_IUnknown)
	{
		*ppvObject = this;
		AddRef();
		return S_OK;
	}

	return E_NOINTERFACE;
}

ULONG CEmWindow::AddRef( void)
{
	return ++m_cRef;
}

ULONG CEmWindow::Release( void)
{
	if (--m_cRef != 0)
		return m_cRef;

	delete this;

	return 0;
}
	
BOOL CEmWindow::PreTranslateMsg(MSG &msg)
{
	return FALSE;
}

extern HRESULT SoundActivate(HWND hwnd, BOOL fActivate);

void OnCmdSound(HWND hwnd)
{
	if (SUCCEEDED(SoundActivate(hwnd, !_fSoundActive)))
		_fSoundActive = !_fSoundActive;
}

static BOOL CALLBACK AboutDlgProc(HWND hwndDlg, UINT uMsg,	WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:
		{
			HWND hwndCtrl = GetWindow(hwndDlg, GW_CHILD);

			while (hwndCtrl != NULL)
			{
				TCHAR szBuf[256];

				if (GetWindowText(hwndCtrl, szBuf, DIM(szBuf)) > 0)
				{
					if (szBuf[0] == TEXT('<') &&
						szBuf[1] == TEXT('v') &&
						szBuf[2] == TEXT('e') &&
						szBuf[3] == TEXT('r') &&
						szBuf[4] == TEXT(':'))
					{
						// TODO:  finish parsing version resources
					}
				}

				hwndCtrl = GetWindow(hwndCtrl, GW_HWNDNEXT);
			}

			return TRUE;
		}

		case WM_COMMAND:
		{
			switch (LOWORD(wParam))
			{
				case IDOK:
					EndDialog(hwndDlg, 0);
					return TRUE;
			}
		}
	}

	return FALSE;
}

static BOOL CALLBACK LicenseDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
/*
	various silly license terms...  one day, I'll display a license of my own that
	outlines my terms...  (maybe a spoof is in order)

	1) you CAN distribute this product for profit if you want to (under GNU copyleft,
	   of course) once we replace Marat's  CPU emulation code with our own, since his
	   license is restricting me at the moment

	2) Don't need to babysit the users about the legalities of distributing ROMs.

	3) Violation of conditions WILL NOT affect the future of this project.  I'm doing
	   it solely because I love these old games.

	4) Not liable, no warranty

  this one's pretty good about the ROM distribution policy...
  
  printf("This work may not be distributed for profit by any means, electronic\n");
  printf("or otherwise. This licensing agreement will be enforced under any\n");
  printf("and all applicable local and/or international copyright laws.\n\n");
  printf("Furthermore, the ROM images required for the effective use of \n");
  printf("this software remain under copyright by their manufacturer. You are \n");
  printf("only licensed to use the ROM images if you own the hardware that \n");
  printf("the images were licensed for use with. \n\n");

   Here's the royal anal-retentive club policy I found...

Conditions of use:

Sparcade may be made freely available from Internet web sites as long as
the original zip file is not modified in any way.  All other forms of
distribution are strictly prohibited.  This restriction includes, but is
not limited to, distribution on floppy disks, CDs or via BBS.  In particular,
attempting to sell Sparcade or distribute it with game ROM code constitutes
a violation of copyright and may result in legal action being taken.

Anyone using this program does so at their own risk.  The author does not
accept any liability for loss or damage arising from its use.

Sparcade may only be used for the purpose of playing the supported games.
Users must not disassemble or reassemble the program code or produce
derivative works based on the concepts or program code of the emulator.

The author will not be held responsible for copyright violations by users of
the emulator.  Users are reminded that the games supported are copyrighted
and should not be distributed without permission.

To use this piece of software, you must agree to be bound by the conditions
of use listed above.  If you cannot agree to these terms, please delete your
copy of the emulator.

Violation of the conditions of use will directly affect the future of the
Sparcade emulation project.

//  The Shinobi emulator makes some lame comment about Windoze, so...
//		"MageX v0.3";
//		"Windows %i.%i detected! MageX dosen't run under DOS!";

eventually, maybe I'll port that guys work to MageX and see if I can give
him a run for his money

all in good fun :-)

*/

	// there...  that's really nice
	
	static const TCHAR szLicense[] =
		"Welcome to version %s of MageX.";

	switch (uMsg)
	{
		case WM_INITDIALOG:
		{
			return TRUE;
		}

		case WM_COMMAND:
		{
			switch (LOWORD(wParam))
			{
				case IDYES:
				case IDNO:
					EndDialog(hwndDlg, LOWORD(wParam));
					return TRUE;
			}
		}
	}

	return FALSE;
}

static BOOL SetCurrentGame(int iNewGame)
{
	_nCurrentGame = iNewGame;
	_pgmdActive = _rgpGameDesc[_nCurrentGame];

	return TRUE;

	_pgmdActive = &_gmdActive;

	_ASSERT(iNewGame < _cGames);

	ZERO_INIT(_gmdActive);

	for (int i = 0; _rgpGameDesc[i] != NULL; i++)
	{
		if (lstrcmpi(_rgGames[iNewGame].szTitle, _rgpGameDesc[i]->pszTitle) == 0)
		{
			_nCurrentGame = i;
			_pgmdActive = _rgpGameDesc[_nCurrentGame];

			return TRUE;
		}
	}

	return FALSE;
}

static BOOL CALLBACK WelcomeDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:
		{
			WndCenter(hwndDlg, GetDesktopWindow());

			CheckDlgButton(hwndDlg, IDC_DISPLAY_ON_STARTUP, BM_SETCHECK);

			if (_glb.fDirectXInstalled)
			{
#ifndef _DEBUG
				if (_glb.fUseDirectX)
				{
					if (_glb.fUseSystemMemory)
						CheckDlgButton(hwndDlg, IDC_DD_SYSMEM, BST_CHECKED);
					else
						CheckDlgButton(hwndDlg, IDC_DD_NORMAL, BST_CHECKED);
				}
				else
#endif
				{
					CheckDlgButton(hwndDlg, IDC_DD_NONE, BST_CHECKED);
				}
			}
			else
			{
				CheckDlgButton(hwndDlg, IDC_DD_NONE, BST_CHECKED);
				
				EnableWindow(GetDlgItem(hwndDlg, IDC_DD_SYSMEM), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_DD_NORMAL), FALSE);
			}
			
			for (int i = 0; _rgpGameDesc[i] != NULL; i++)
			{
				SendDlgItemMessage(hwndDlg, IDC_GAME, CB_ADDSTRING,
					0, (LPARAM)_rgpGameDesc[i]->pszTitle);
			}

/*
			for (int i = 0; i < _cGames; i++)
			{
				SendDlgItemMessage(hwndDlg, IDC_GAME, CB_ADDSTRING,
					0, (LPARAM)_rgGames[i].szTitle);
			}
*/

			SendDlgItemMessage(hwndDlg, IDC_GAME, CB_SETCURSEL, _nCurrentGame, 0);

			return TRUE;
		}

		case WM_COMMAND:
		{
			switch (LOWORD(wParam))
			{
				case IDOK:
				{
					if (IsDlgButtonChecked(hwndDlg, IDC_DD_NONE))
					{
						_glb.fUseDirectX = FALSE;
						_glb.fUseSystemMemory = FALSE;
						_glb.fUseVideoMemory = FALSE;
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_DD_NORMAL))
					{
						_glb.fUseDirectX = TRUE;
						_glb.fUseSystemMemory = FALSE;
						_glb.fUseVideoMemory = FALSE;
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_DD_SYSMEM))
					{
						_glb.fUseDirectX = TRUE;
						_glb.fUseSystemMemory = TRUE;
						_glb.fUseVideoMemory = FALSE;
					}

					_fJoystickEnable = IsDlgButtonChecked(hwndDlg, IDC_JOYSTICK);

					int i = SendDlgItemMessage(hwndDlg, IDC_GAME, CB_GETCURSEL, 0, 0);

					if (i != CB_ERR)
					{
						SetCurrentGame(i);
					}
				}

				case IDCANCEL:
					EndDialog(hwndDlg, LOWORD(wParam));
					return TRUE;
			}
		}
	}

	return FALSE;
}

//
//  Execute commands
//

BOOL OnCommand(HWND hwnd, UINT nId)
{
	//
	//  First, route command through active game
	//
	
	if (GetActiveGame()->pfnCmd != NULL)
	{
		if (GetActiveGame()->pfnCmd(hwnd, nId) != S_FALSE)
			return TRUE;
	}

	//
	//  Has someone selected a new game?
	//

	if ((nId >= ID_GAMEBASE) && (nId <= ID_GAMEMAX))
	{
		int iGame = nId - ID_GAMEBASE;
		
		if (_nCurrentGame == iGame)
		{
			if (MessageBox(hwnd, TEXT("There is a game currently in progress.  Would you like to reset this game?"),
				"Reset Game?", MB_ICONWARNING | MB_OKCANCEL) != IDOK)
			{
				return TRUE;
			}
		}
		else
		{
			if (MessageBox(hwnd, TStr<128>("There is a game currently in progress.  Would you like to switch to %s",
				_rgpGameDesc[iGame]->pszTitle),
//				_rgGames[iGame].szTitle),
				"Switch Game?", MB_ICONWARNING | MB_OKCANCEL) != IDOK)
			{
				return TRUE;
			}
		}

		SetCurrentGame(iGame);

/*
		_nCurrentGame = iGame;
		_pgmdActive = _rgpGameDesc[_nCurrentGame];
*/

		ZERO_INIT(RAM); // HACK!!!  some games AVF

		PostQuitMessage(0);

		return TRUE;
	}

	if ((nId >= ID_MODEBASE) && (nId <= ID_MODEMAX))
	{
		int iMode = nId - ID_MODEBASE;

		if (_lpdd != NULL)
		{
			if (SUCCEEDED(_lpdd->SetDisplayMode(_rgsizMode[iMode].cx,
				_rgsizMode[iMode].cy, _ddsd.ddpfPixelFormat.dwRGBBitCount)))
			{
//				GAME_LOAD_INFO gli;
				
				if (SUCCEEDED(CreateObjs(_lpdd, _rgpe, hwnd, _gli)))
				{
					_iMode = iMode;
					_ddsd.dwWidth = _rgsizMode[iMode].cx;
					_ddsd.dwHeight = _rgsizMode[iMode].cy;
				}
			}
			else
			{
			}
		}
		
		return TRUE;
	}

	//
	//
	//

	switch (nId)
	{
		case ID_FILE_EXIT:
			SendMessage(hwnd, WM_CLOSE, 0, 0);
			return TRUE;

		case ID_EMULATOR_SOUND:
			OnCmdSound(hwnd);
			return TRUE;

		case ID_RENDER_NOP:
			_pfnIdle = IdleNop;
			return TRUE;

		case ID_RENDER_BLITONLY:
			_pfnIdle = IdleBlitOnly;
			return TRUE;

		case ID_RENDER_0X0:
			_nBlitMode = 0;
			return TRUE;

		case ID_RENDER_1X1:
			_nBlitMode = 1;
			return TRUE;

		case ID_RENDER_2X2:
			_nBlitMode = 2;
			return TRUE;

		case ID_RENDER_FULL:
			_pfnIdle = IdleNormal;
			return TRUE;

		case ID_RENDER_CHARSET:
			_pfnIdle = IdleCharset;
			return TRUE;

		case ID_RENDER_MEMORY:
			_pfnIdle = IdleShowMem;
			return TRUE;

		case ID_HELP_ABOUT:
			DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ABOUT),
				hwnd, (DLGPROC)AboutDlgProc);
			return TRUE;

		case ID_OPTIONS_THROTTLE:
			_fThrottle = !_fThrottle;
			return TRUE;

		case ID_CPU_STATS:
			_fCpuStats = !_fCpuStats;
			return TRUE;

		case ID_OPTIONS_JOYSTICK:
			_fJoystickEnable = !_fJoystickEnable;
			break;

		case ID_OPTIONS_VBLANK:
			_fWaitVBlank = !_fWaitVBlank;
			break;
	}

	return FALSE;
}

LRESULT CEmWindow::OnMessage(MSG &msg)
{
	switch (msg.message)
	{
		case WM_NCCREATE:
		{
//			m_hwnd = msg.hwnd;
			break;
		}

		case WM_NCDESTROY:
		{
//			m_hwnd = NULL;
			break;
		}

		case WM_CREATE:
		{
			WndCenter(msg.hwnd, GetDesktopWindow());

			_hdc = GetDC(msg.hwnd);

			BuildFileMenu(msg.hwnd);

			//
			//
			//
			
			DefWindowProc(msg.hwnd, WM_SETICON, TRUE,
				(LPARAM)LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_MAIN)));

			//
			//  Set timer to recalc frame rate every second
			//
			
			SetTimer(msg.hwnd, 1, 1000, NULL);

			break;
		}

		case WM_COMMAND:
		{
			if (OnCommand(msg.hwnd, LOWORD(msg.wParam)))
				return 0;

			break;
		}

		case WM_ERASEBKGND:
		{
			RECT rcClient;

			GetClientRect(msg.hwnd, &rcClient);

			FillRect((HDC)msg.wParam, &rcClient, GetStockObject(BLACK_BRUSH));

			return 0;
		}

		case WM_PAINT:
		{
			PAINTSTRUCT ps;

			BeginPaint(msg.hwnd, &ps);

			if (_hdc != NULL)
			{
				RealizePalette(_hdc);
			}

			DoBlit();

			EndPaint(msg.hwnd, &ps);

			break;
		}

		case WM_PALETTECHANGED:
		{
			if ((HWND)msg.wParam == msg.hwnd)
				break;
		}

		// Else fall through...

		case WM_QUERYNEWPALETTE:
		{
			if (_pddsPrimary != NULL)
			{
				_pddsPrimary->SetPalette(_pddPalette);
			}
			else
			{
//				HPALETTE hOldPal = SelectPalette(_hdc, _hpal, FALSE);

				int i = RealizePalette(_hdc);
				
				if (i != 0)
					InvalidateRect(msg.hwnd, NULL, TRUE); 

//				SelectPalette(_hdc, hOldPal, TRUE);

				return i;
			}

			break;
		}

		case WM_TIMER:
		{
			if (msg.wParam == 1)
			{
				//
				//  Calculate FPS
				//

				DWORD dwNow = timeGetTime();
				_cFps = MulDiv(cFrame, 1000, dwNow - dwTick);
				cFrame = 0;
				dwTick = dwNow;

				SetWindowText(msg.hwnd, TStr<128>(TITLE " - %s (%i fps)",
					GetActiveGame()->pszTitle, _cFps));
			}

			break;
		}

		case WM_CLOSE:
		{
			_fQuitProgram = TRUE;
			PostQuitMessage(0);
			return 0;
		}

		case WM_DESTROY:
		{
			if (_hpal != NULL)
				DeleteObject(_hpal);
			
			if (_hbmp != NULL)
				DeleteObject(_hbmp);
			
			if (_pbmi != NULL)
				free(_pbmi);
			
			KillTimer(msg.hwnd, 1);
			
			if (_hdc != NULL)			
				ReleaseDC(msg.hwnd, _hdc);

			if (_fSoundActive)
				OnCmdSound(msg.hwnd);

			break;
		}

#if 0
		case WM_SETCURSOR:
		{
			/*
			 * Prevent the cursor from being shown in fullscreen
			 */
			if (d3dappi.bFullscreen && !d3dappi.bPaused) {
				SetCursor(NULL);
				*lresult = 1;
				*bStopProcessing = TRUE;
				return TRUE;
			}
			break;
		}

		case WM_MOVING:
			/*
			 * Prevent the window from moving in fullscreen
			 */
			if (d3dappi.bFullscreen) {
				GetWindowRect(hwnd, (LPRECT)lParam);
				*lresult = 1;
				*bStopProcessing = TRUE;
				return TRUE;
			}
			break;
		case WM_PAINT:
			/*
			 * Clear the rectangle and blt the backbuffer image
			 */
			BeginPaint(hwnd, &ps);
			if (d3dappi.bRenderingIsOK && !d3dappi.bFullscreen) {
				D3DAppShowBackBuffer(D3DAPP_SHOWALL);
			}
			D3DAppIValidateDirtyRects();
			EndPaint(hwnd, &ps);
			*lresult = 1;
			*bStopProcessing = TRUE;
			return TRUE;
		case WM_GETMINMAXINFO:
		{
			((LPMINMAXINFO)lParam)->ptMaxTrackSize.x= _ddsd.dwWidth;
			((LPMINMAXINFO)lParam)->ptMaxTrackSize.y= _ddsd.dwHeight;
			((LPMINMAXINFO)lParam)->ptMinTrackSize.x= _ddsd.dwWidth;
			((LPMINMAXINFO)lParam)->ptMinTrackSize.y= _ddsd.dwHeight;
			return 0;
		}
#endif

		case WM_NCPAINT:
		{
			if (_pddsPrimary != NULL)
				return 0;

			break;
		}

		case WM_KEYDOWN:
		{
			if (msg.wParam == VK_ESCAPE)
				PostQuitMessage(0);
			
			break;
		}
	}

	return DefWindowProc(msg.hwnd, msg.message, msg.wParam, msg.lParam);
}

/*---------------------------------------------------------------------------

	BuildFileMenu - slight varation of code contributed by Ivan Mackintosh

---------------------------------------------------------------------------*/

void BuildFileMenu(HWND hwnd)
{
	int i;
	MENUITEMINFO mii;

	HMENU hmenu = GetSubMenu(GetMenu(hwnd), 0);

	mii.cbSize        = sizeof(mii);
	
	//
	//  First, insert the separator
	//
	
	mii.fMask         = MIIM_TYPE;
	mii.fType         = MFT_SEPARATOR;

	InsertMenuItem(hmenu, 0, TRUE, &mii);

	int cItems = GetMenuItemCount(hmenu);

	//
	//
	//

	mii.fType = MFT_STRING;
	mii.fMask = MIIM_TYPE | MIIM_ID;

	for (i = 0; _rgpGameDesc[i] != NULL; i++)
//	for (i = 0; i < _cGames; i++)
	{
		mii.wID        = ID_GAMEBASE + i;
		mii.dwTypeData = const_cast<LPTSTR>(_rgpGameDesc[i]->pszTitle);
//		mii.dwTypeData = const_cast<LPTSTR>(_rgGames[i].szTitle);
		mii.cch        = lstrlen(mii.dwTypeData);
		
		InsertMenuItem(hmenu, GetMenuItemCount(hmenu) - cItems, TRUE, &mii);
	}
}

//
//  Handle UI updates of commands
//

void CEmWindow::UpdateUI(int nCmdId, UINT &nState)
{
	if (GetActiveGame()->pfnCmdUpdate != NULL)
	{
		if (GetActiveGame()->pfnCmdUpdate(nCmdId, nState) != S_FALSE)
			return;
	}

	//
	//
	//

	if ((nCmdId >= ID_GAMEBASE) && (nCmdId <= ID_GAMEMAX))
	{
		if (_nCurrentGame == (nCmdId - ID_GAMEBASE))
			nState |= MFS_CHECKED;

		return;
	}

	//
	//
	//

	if ((nCmdId >= ID_MODEBASE) && (nCmdId <= ID_MODEMAX))
	{
		// hack to disable 320x200 and 320x240 modes since they're not working on all cards
/*
		if (_rgsizMode[nCmdId - ID_MODEBASE].cx == 320)
			nState |= MFS_DISABLED;
*/

		if (_iMode == (nCmdId - ID_MODEBASE))
			nState |= MFS_CHECKED;

		return;
	}

	switch (nCmdId)
	{
		case ID_EMULATOR_SOUND:
			
			if (waveOutGetNumDevs() > 0)
				nState |= (_fSoundActive ? MFS_CHECKED : 0);
			
			break;

		case ID_RENDER_NOP:
			nState |= (_pfnIdle == IdleNop) ? MFS_CHECKED : 0;
			break;

		case ID_RENDER_BLITONLY:
			nState |= (_pfnIdle == IdleBlitOnly) ? MFS_CHECKED : 0;
			break;

		case ID_RENDER_0X0:
			nState |= (_nBlitMode == 0) ? MFS_CHECKED : 0;
			break;

		case ID_RENDER_1X1:
			nState |= (_nBlitMode == 1) ? MFS_CHECKED : 0;
			break;

		case ID_RENDER_2X2:
			nState |= (_nBlitMode == 2) ? MFS_CHECKED : 0;
			break;

		case ID_RENDER_FULL:
			nState |= (_pfnIdle == IdleNormal) ? MFS_CHECKED : 0;
			break;

		case ID_RENDER_CHARSET:
			nState |= (_pfnIdle == IdleCharset) ? MFS_CHECKED : 0;
			break;

		case ID_RENDER_MEMORY:
			nState |= (_pfnIdle == IdleShowMem) ? MFS_CHECKED : 0;
			break;

		case ID_FILE_EXIT:
		case ID_HELP_ABOUT:
			break;

		case ID_OPTIONS_THROTTLE:
			nState |= (_fThrottle) ? MFS_CHECKED : 0;
			break;

		case ID_CPU_STATS:
			nState |= (_fCpuStats) ? MFS_CHECKED : 0;
			break;

		case ID_OPTIONS_JOYSTICK:
			nState |= (_fJoystickEnable) ? MFS_CHECKED : 0;
			break;

		case ID_OPTIONS_VBLANK:
			if (_lpdd == NULL)
				nState |= MFS_DISABLED;
			else
				nState |= (_fWaitVBlank) ? MFS_CHECKED : 0;
			break;

		default:
			nState |= MFS_DISABLED;
			break;
	}
}

static DWORD _dwPrevTime;
extern int _fThrottle;

static int _nRate = 100;

DWORD PollJoystick();
static DWORD _dwStick = 0;

// CONSIDER:  check messages inside the loop?
// TODO:  poll joystick only when time has expired (20 ms)
// CONSIDER:  use Sleep if we've got enough time for a nap

static BYTE _rgbKeyboardState[256];

static void Throttle(int fps)
{
	GetKeyboardState(_rgbKeyboardState);

	if (_fJoystickEnable)
		_dwStick = PollJoystick();

	if (_fThrottle)
	{
		//
		//
		//

		DWORD dwTarget = (1000 / MulDiv(fps, _nRate, 100));

		if (GetAsyncKeyState(VK_SUBTRACT) < 0 && _nRate > 0)
			_nRate--;
		else if (GetAsyncKeyState(VK_ADD) < 0)
			_nRate++;

		//
		//  Use the high frequency timer if available
		//
		
		LARGE_INTEGER nFreq, nCurrTime;
		
		if (QueryPerformanceFrequency(&nFreq))
		{
			static LARGE_INTEGER nPrevTime, nTargTime;

			for (;;)
			{
				if (!QueryPerformanceCounter(&nCurrTime))
					break;

				if (nCurrTime.QuadPart >= nTargTime.QuadPart)
					break;
			}

			nPrevTime = nCurrTime;
			nTargTime.QuadPart = nCurrTime.QuadPart + nFreq.QuadPart / MulDiv(fps, _nRate, 100);
		}
		else
		{
			DWORD dwCurrTime;

			for (;;)
			{
				dwCurrTime = timeGetTime();

				if ((dwCurrTime - _dwPrevTime) >= dwTarget)
					break;
			}

			_dwPrevTime = dwCurrTime;
		}
	}
}

static DWORD PollJoystick()
{
	DWORD dwResult = 0;
	JOYINFOEX ji;

	ji.dwSize = sizeof(ji);
	ji.dwFlags = JOY_RETURNX | JOY_RETURNY | JOY_RETURNBUTTONS;

	if (joyGetPosEx(JOYSTICKID1, &ji) == JOYERR_NOERROR)
	{
		if (ji.dwButtons & JOY_BUTTON1)
			dwResult |= 0x00000001;

		if (ji.dwButtons & JOY_BUTTON2)
			dwResult |= 0x00000002;

		if (ji.dwYpos < 16384)
			dwResult |= 0x00000004;

		if (ji.dwYpos > 65535 - 16384)
			dwResult |= 0x00000008;

		if (ji.dwXpos < 16384)
			dwResult |= 0x00000010;

		if (ji.dwXpos > 65535 - 16384)
			dwResult |= 0x00000020;
	}

	return dwResult;
}

int osd_key_pressed(int keycode)
{
	if (keycode == 0)
		return FALSE;

	if (keycode == OSD_KEY_ALT) // hack until we can use Alt key since Windows needs it
		keycode = OSD_KEY_Z;

	//
	//  OSD_KEY_ constants are actually scan codes which must be
	//  converted into virtual key codes for use here
	//
	
//	if (GetAsyncKeyState(MapVirtualKey(keycode, 1)) < 0)
	if (_rgbKeyboardState[MapVirtualKey(keycode, 1)] & 0x80)
		return 1;

	//
	//  
	//

	if (_fJoystickEnable)
	{
		switch (keycode)
		{
			case OSD_JOYSTICK_B1:
			case OSD_KEY_CONTROL:
				return _dwStick & 0x00000001;

			case OSD_KEY_LSHIFT:
			case OSD_KEY_RSHIFT:
			case OSD_KEY_ALT:
			case OSD_KEY_Z:
			case OSD_JOYSTICK_B2:
				return _dwStick & 0x00000002;

			case OSD_KEY_UP:
				return _dwStick & 0x00000004;

			case OSD_KEY_DOWN:
				return _dwStick & 0x00000008;

			case OSD_KEY_LEFT:
				return _dwStick & 0x00000010;

			case OSD_KEY_RIGHT:
				return _dwStick & 0x00000020;
		}
	}

	return 0;
}

